@@ -1,8 +1,6 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * SPDX-License-Identifier: MPL-2.0
- *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, you can obtain one at https://mozilla.org/MPL/2.0/.
@@ -36,7 +34,6 @@
 #include <dns/events.h>
 #include <dns/log.h>
 #include <dns/message.h>
-#include <dns/portlist.h>
 #include <dns/stats.h>
 #include <dns/tcpmsg.h>
 #include <dns/types.h>
@@ -63,7 +60,6 @@ struct dns_dispatchmgr {
 	unsigned int magic;
 	isc_mem_t *mctx;
 	dns_acl_t *blackhole;
-	dns_portlist_t *portlist;
 	isc_stats_t *stats;
 
 	/* Locked by "lock". */
@@ -78,7 +74,17 @@ struct dns_dispatchmgr {
 	unsigned int buffersize; /*%< size of each buffer */
 	unsigned int maxbuffers; /*%< max buffers */
 
-	isc_refcount_t irefs;
+	/* Locked internally. */
+	isc_mutex_t depool_lock;
+	isc_mempool_t *depool; /*%< pool for dispatch events */
+	isc_mutex_t rpool_lock;
+	isc_mempool_t *rpool; /*%< pool for replies */
+	isc_mutex_t dpool_lock;
+	isc_mempool_t *dpool; /*%< dispatch allocations */
+	isc_mutex_t bpool_lock;
+	isc_mempool_t *bpool; /*%< pool for buffers */
+	isc_mutex_t spool_lock;
+	isc_mempool_t *spool; /*%< pool for dispsocks */
 
 	/*%
 	 * Locked by qid->lock if qid exists; otherwise, can be used without
@@ -200,7 +206,8 @@ struct dns_dispatch {
 	unsigned int maxrequests; /*%< max requests */
 	isc_event_t *ctlevent;
 
-	isc_mem_t *sepool; /*%< pool for socket events */
+	isc_mutex_t sepool_lock;
+	isc_mempool_t *sepool; /*%< pool for socket events */
 
 	/*% Locked by mgr->lock. */
 	ISC_LINK(dns_dispatch_t) link;
@@ -225,6 +232,7 @@ struct dns_dispatch {
 	dns_tcpmsg_t tcpmsg;	 /*%< for tcp streams */
 	dns_qid_t *qid;
 	dispportlist_t *port_table; /*%< hold ports 'owned' by us */
+	isc_mempool_t *portpool;    /*%< port table entries  */
 };
 
 #define QID_MAGIC    ISC_MAGIC('Q', 'i', 'd', ' ')
@@ -290,9 +298,9 @@ static void
 free_buffer(dns_dispatch_t *disp, void *buf, unsigned int len);
 static void *
 allocate_udp_buffer(dns_dispatch_t *disp);
-static void
+static inline void
 free_devent(dns_dispatch_t *disp, dns_dispatchevent_t *ev);
-static dns_dispatchevent_t *
+static inline dns_dispatchevent_t *
 allocate_devent(dns_dispatch_t *disp);
 static void
 do_cancel(dns_dispatch_t *disp);
@@ -352,14 +360,14 @@ mgr_log(dns_dispatchmgr_t *mgr, int level, const char *fmt, ...) {
 		      msgbuf);
 }
 
-static void
+static inline void
 inc_stats(dns_dispatchmgr_t *mgr, isc_statscounter_t counter) {
 	if (mgr->stats != NULL) {
 		isc_stats_increment(mgr->stats, counter);
 	}
 }
 
-static void
+static inline void
 dec_stats(dns_dispatchmgr_t *mgr, isc_statscounter_t counter) {
 	if (mgr->stats != NULL) {
 		isc_stats_decrement(mgr->stats, counter);
@@ -539,7 +547,8 @@ destroy_disp(isc_task_t *task, isc_event_t *event) {
 		     disp->socket, disp->task[0]); /* XXXX */
 
 	if (disp->sepool != NULL) {
-		isc_mem_destroy(&disp->sepool);
+		isc_mempool_destroy(&disp->sepool);
+		isc_mutex_destroy(&disp->sepool_lock);
 	}
 
 	if (disp->socket != NULL) {
@@ -592,7 +601,10 @@ new_portentry(dns_dispatch_t *disp, in_port_t port) {
 
 	REQUIRE(disp->port_table != NULL);
 
-	portentry = isc_mem_get(disp->mgr->mctx, sizeof(*portentry));
+	portentry = isc_mempool_get(disp->portpool);
+	if (portentry == NULL) {
+		return (portentry);
+	}
 
 	portentry->port = port;
 	isc_refcount_init(&portentry->refs, 1);
@@ -621,7 +633,7 @@ deref_portentry(dns_dispatch_t *disp, dispportentry_t **portentryp) {
 		ISC_LIST_UNLINK(disp->port_table[portentry->port %
 						 DNS_DISPATCH_PORTTABLESIZE],
 				portentry, link);
-		isc_mem_put(disp->mgr->mctx, portentry, sizeof(*portentry));
+		isc_mempool_put(disp->portpool, portentry);
 	}
 }
 
@@ -691,7 +703,10 @@ get_dispsocket(dns_dispatch_t *disp, const isc_sockaddr_t *dest,
 		sock = dispsock->socket;
 		dispsock->socket = NULL;
 	} else {
-		dispsock = isc_mem_get(mgr->mctx, sizeof(*dispsock));
+		dispsock = isc_mempool_get(mgr->spool);
+		if (dispsock == NULL) {
+			return (ISC_R_NOMEMORY);
+		}
 
 		disp->nsockets++;
 		dispsock->socket = NULL;
@@ -817,7 +832,7 @@ destroy_dispsocket(dns_dispatch_t *disp, dispsocket_t **dispsockp) {
 	if (dispsock->task != NULL) {
 		isc_task_detach(&dispsock->task);
 	}
-	isc_mem_put(disp->mgr->mctx, dispsock, sizeof(*dispsock));
+	isc_mempool_put(disp->mgr->spool, dispsock);
 }
 
 /*%
@@ -885,8 +900,7 @@ entry_search(dns_qid_t *qid, const isc_sockaddr_t *dest, dns_messageid_t id,
 
 	while (res != NULL) {
 		if (res->id == id && isc_sockaddr_equal(dest, &res->host) &&
-		    res->port == port)
-		{
+		    res->port == port) {
 			return (res);
 		}
 		res = ISC_LIST_NEXT(res, link);
@@ -897,7 +911,7 @@ entry_search(dns_qid_t *qid, const isc_sockaddr_t *dest, dns_messageid_t id,
 
 static void
 free_buffer(dns_dispatch_t *disp, void *buf, unsigned int len) {
-	unsigned int buffersize;
+	isc_mempool_t *bpool;
 	INSIST(buf != NULL && len != 0);
 
 	switch (disp->socktype) {
@@ -911,45 +925,58 @@ free_buffer(dns_dispatch_t *disp, void *buf, unsigned int len) {
 		INSIST(disp->mgr->buffers > 0);
 		INSIST(len == disp->mgr->buffersize);
 		disp->mgr->buffers--;
-		buffersize = disp->mgr->buffersize;
+		bpool = disp->mgr->bpool;
 		UNLOCK(&disp->mgr->buffer_lock);
-		isc_mem_put(disp->mgr->mctx, buf, buffersize);
+		isc_mempool_put(bpool, buf);
 		break;
 	default:
-		UNREACHABLE();
+		INSIST(0);
+		ISC_UNREACHABLE();
 	}
 }
 
 static void *
 allocate_udp_buffer(dns_dispatch_t *disp) {
-	unsigned int buffersize;
+	isc_mempool_t *bpool;
+	void *temp;
 
 	LOCK(&disp->mgr->buffer_lock);
 	if (disp->mgr->buffers >= disp->mgr->maxbuffers) {
 		UNLOCK(&disp->mgr->buffer_lock);
 		return (NULL);
 	}
-	buffersize = disp->mgr->buffersize;
+	bpool = disp->mgr->bpool;
 	disp->mgr->buffers++;
 	UNLOCK(&disp->mgr->buffer_lock);
 
-	return (isc_mem_get(disp->mgr->mctx, buffersize));
+	temp = isc_mempool_get(bpool);
+
+	if (temp == NULL) {
+		LOCK(&disp->mgr->buffer_lock);
+		disp->mgr->buffers--;
+		UNLOCK(&disp->mgr->buffer_lock);
+	}
+
+	return (temp);
 }
 
-static void
+static inline void
 free_sevent(isc_event_t *ev) {
-	isc_mem_t *pool = ev->ev_destroy_arg;
+	isc_mempool_t *pool = ev->ev_destroy_arg;
 	isc_socketevent_t *sev = (isc_socketevent_t *)ev;
-	isc_mem_put(pool, sev, sizeof(*sev));
+	isc_mempool_put(pool, sev);
 }
 
-static isc_socketevent_t *
+static inline isc_socketevent_t *
 allocate_sevent(dns_dispatch_t *disp, isc_socket_t *sock, isc_eventtype_t type,
 		isc_taskaction_t action, const void *arg) {
 	isc_socketevent_t *ev;
 	void *deconst_arg;
 
-	ev = isc_mem_get(disp->sepool, sizeof(*ev));
+	ev = isc_mempool_get(disp->sepool);
+	if (ev == NULL) {
+		return (NULL);
+	}
 	DE_CONST(arg, deconst_arg);
 	ISC_EVENT_INIT(ev, sizeof(*ev), 0, NULL, type, action, deconst_arg,
 		       sock, free_sevent, disp->sepool);
@@ -963,7 +990,7 @@ allocate_sevent(dns_dispatch_t *disp, isc_socket_t *sock, isc_eventtype_t type,
 	return (ev);
 }
 
-static void
+static inline void
 free_devent(dns_dispatch_t *disp, dns_dispatchevent_t *ev) {
 	if (disp->failsafe_ev == ev) {
 		INSIST(disp->shutdown_out == 1);
@@ -972,16 +999,17 @@ free_devent(dns_dispatch_t *disp, dns_dispatchevent_t *ev) {
 		return;
 	}
 
-	isc_refcount_decrement(&disp->mgr->irefs);
-	isc_mem_put(disp->mgr->mctx, ev, sizeof(*ev));
+	isc_mempool_put(disp->mgr->depool, ev);
 }
 
-static dns_dispatchevent_t *
+static inline dns_dispatchevent_t *
 allocate_devent(dns_dispatch_t *disp) {
 	dns_dispatchevent_t *ev;
 
-	ev = isc_mem_get(disp->mgr->mctx, sizeof(*ev));
-	isc_refcount_increment0(&disp->mgr->irefs);
+	ev = isc_mempool_get(disp->mgr->depool);
+	if (ev == NULL) {
+		return (NULL);
+	}
 	ISC_EVENT_INIT(ev, sizeof(*ev), 0, NULL, 0, NULL, NULL, NULL, NULL,
 		       NULL);
 
@@ -1189,9 +1217,13 @@ udp_recv(isc_event_t *ev_in, dns_dispatch_t *disp, dispsocket_t *dispsock) {
 			     "search for response in bucket %d: %s", bucket,
 			     (resp == NULL ? "not found" : "found"));
 
+		if (resp == NULL) {
+			inc_stats(mgr, dns_resstatscounter_mismatch);
+			free_buffer(disp, ev->region.base, ev->region.length);
+			goto unlock;
+		}
 	} else if (resp->id != id ||
-		   !isc_sockaddr_equal(&ev->address, &resp->host))
-	{
+		   !isc_sockaddr_equal(&ev->address, &resp->host)) {
 		dispatch_log(disp, LVL(90),
 			     "response to an exclusive socket doesn't match");
 		inc_stats(mgr, dns_resstatscounter_mismatch);
@@ -1199,12 +1231,6 @@ udp_recv(isc_event_t *ev_in, dns_dispatch_t *disp, dispsocket_t *dispsock) {
 		goto unlock;
 	}
 
-	if (resp == NULL) {
-		inc_stats(mgr, dns_resstatscounter_mismatch);
-		free_buffer(disp, ev->region.base, ev->region.length);
-		goto unlock;
-	}
-
 	/*
 	 * Now that we have the original dispatch the query was sent
 	 * from check that the address and port the response was
@@ -1515,8 +1541,7 @@ startrecv(dns_dispatch_t *disp, dispsocket_t *dispsock) {
 	}
 
 	if ((disp->attributes & DNS_DISPATCHATTR_EXCLUSIVE) != 0 &&
-	    dispsock == NULL)
-	{
+	    dispsock == NULL) {
 		return (ISC_R_SUCCESS);
 	}
 
@@ -1588,7 +1613,8 @@ startrecv(dns_dispatch_t *disp, dispsocket_t *dispsock) {
 		disp->recv_pending = 1;
 		break;
 	default:
-		UNREACHABLE();
+		INSIST(0);
+		ISC_UNREACHABLE();
 	}
 
 	return (ISC_R_SUCCESS);
@@ -1600,15 +1626,25 @@ startrecv(dns_dispatch_t *disp, dispsocket_t *dispsock) {
 static bool
 destroy_mgr_ok(dns_dispatchmgr_t *mgr) {
 	mgr_log(mgr, LVL(90),
-		"destroy_mgr_ok: shuttingdown=%d, listnonempty=%d, ",
-		MGR_IS_SHUTTINGDOWN(mgr), !ISC_LIST_EMPTY(mgr->list));
+		"destroy_mgr_ok: shuttingdown=%d, listnonempty=%d, "
+		"depool=%d, rpool=%d, dpool=%d",
+		MGR_IS_SHUTTINGDOWN(mgr), !ISC_LIST_EMPTY(mgr->list),
+		isc_mempool_getallocated(mgr->depool),
+		isc_mempool_getallocated(mgr->rpool),
+		isc_mempool_getallocated(mgr->dpool));
 	if (!MGR_IS_SHUTTINGDOWN(mgr)) {
 		return (false);
 	}
 	if (!ISC_LIST_EMPTY(mgr->list)) {
 		return (false);
 	}
-	if (isc_refcount_current(&mgr->irefs) != 0) {
+	if (isc_mempool_getallocated(mgr->depool) != 0) {
+		return (false);
+	}
+	if (isc_mempool_getallocated(mgr->rpool) != 0) {
+		return (false);
+	}
+	if (isc_mempool_getallocated(mgr->dpool) != 0) {
 		return (false);
 	}
 
@@ -1629,6 +1665,22 @@ destroy_mgr(dns_dispatchmgr_t **mgrp) {
 	isc_mutex_destroy(&mgr->lock);
 	mgr->state = 0;
 
+	isc_mempool_destroy(&mgr->depool);
+	isc_mempool_destroy(&mgr->rpool);
+	isc_mempool_destroy(&mgr->dpool);
+	if (mgr->bpool != NULL) {
+		isc_mempool_destroy(&mgr->bpool);
+	}
+	if (mgr->spool != NULL) {
+		isc_mempool_destroy(&mgr->spool);
+	}
+
+	isc_mutex_destroy(&mgr->spool_lock);
+	isc_mutex_destroy(&mgr->bpool_lock);
+	isc_mutex_destroy(&mgr->dpool_lock);
+	isc_mutex_destroy(&mgr->rpool_lock);
+	isc_mutex_destroy(&mgr->depool_lock);
+
 	if (mgr->qid != NULL) {
 		qid_destroy(mgr->mctx, &mgr->qid);
 	}
@@ -1668,8 +1720,7 @@ open_socket(isc_socketmgr_t *mgr, const isc_sockaddr_t *local,
 			return (result);
 		}
 	} else if (dup_socket != NULL &&
-		   (!isc_socket_hasreuseport() || duponly))
-	{
+		   (!isc_socket_hasreuseport() || duponly)) {
 		result = isc_socket_dup(dup_socket, &sock);
 		if (result != ISC_R_SUCCESS) {
 			return (result);
@@ -1739,17 +1790,61 @@ dns_dispatchmgr_create(isc_mem_t *mctx, dns_dispatchmgr_t **mgrp) {
 	REQUIRE(mgrp != NULL && *mgrp == NULL);
 
 	mgr = isc_mem_get(mctx, sizeof(dns_dispatchmgr_t));
-	*mgr = (dns_dispatchmgr_t){ 0 };
 
+	mgr->mctx = NULL;
 	isc_mem_attach(mctx, &mgr->mctx);
 
+	mgr->blackhole = NULL;
+	mgr->stats = NULL;
+
 	isc_mutex_init(&mgr->lock);
 	isc_mutex_init(&mgr->buffer_lock);
+	isc_mutex_init(&mgr->depool_lock);
+	isc_mutex_init(&mgr->rpool_lock);
+	isc_mutex_init(&mgr->dpool_lock);
+	isc_mutex_init(&mgr->bpool_lock);
+	isc_mutex_init(&mgr->spool_lock);
 
-	isc_refcount_init(&mgr->irefs, 0);
+	mgr->depool = NULL;
+	isc_mempool_create(mgr->mctx, sizeof(dns_dispatchevent_t),
+			   &mgr->depool);
 
+	mgr->rpool = NULL;
+	isc_mempool_create(mgr->mctx, sizeof(dns_dispentry_t), &mgr->rpool);
+
+	mgr->dpool = NULL;
+	isc_mempool_create(mgr->mctx, sizeof(dns_dispatch_t), &mgr->dpool);
+
+	isc_mempool_setname(mgr->depool, "dispmgr_depool");
+	isc_mempool_setmaxalloc(mgr->depool, 32768);
+	isc_mempool_setfreemax(mgr->depool, 32768);
+	isc_mempool_associatelock(mgr->depool, &mgr->depool_lock);
+	isc_mempool_setfillcount(mgr->depool, 32);
+
+	isc_mempool_setname(mgr->rpool, "dispmgr_rpool");
+	isc_mempool_setmaxalloc(mgr->rpool, 32768);
+	isc_mempool_setfreemax(mgr->rpool, 32768);
+	isc_mempool_associatelock(mgr->rpool, &mgr->rpool_lock);
+	isc_mempool_setfillcount(mgr->rpool, 32);
+
+	isc_mempool_setname(mgr->dpool, "dispmgr_dpool");
+	isc_mempool_setmaxalloc(mgr->dpool, 32768);
+	isc_mempool_setfreemax(mgr->dpool, 32768);
+	isc_mempool_associatelock(mgr->dpool, &mgr->dpool_lock);
+	isc_mempool_setfillcount(mgr->dpool, 32);
+
+	mgr->buffers = 0;
+	mgr->buffersize = 0;
+	mgr->maxbuffers = 0;
+	mgr->bpool = NULL;
+	mgr->spool = NULL;
+	mgr->qid = NULL;
+	mgr->state = 0;
 	ISC_LIST_INIT(mgr->list);
-
+	mgr->v4ports = NULL;
+	mgr->v6ports = NULL;
+	mgr->nv4ports = 0;
+	mgr->nv6ports = 0;
 	mgr->magic = DNS_DISPATCHMGR_MAGIC;
 
 	result = create_default_portset(mctx, &v4portset);
@@ -1774,6 +1869,14 @@ dns_dispatchmgr_create(isc_mem_t *mctx, dns_dispatchmgr_t **mgrp) {
 	return (ISC_R_SUCCESS);
 
 kill_dpool:
+	isc_mempool_destroy(&mgr->dpool);
+	isc_mempool_destroy(&mgr->rpool);
+	isc_mempool_destroy(&mgr->depool);
+	isc_mutex_destroy(&mgr->spool_lock);
+	isc_mutex_destroy(&mgr->bpool_lock);
+	isc_mutex_destroy(&mgr->dpool_lock);
+	isc_mutex_destroy(&mgr->rpool_lock);
+	isc_mutex_destroy(&mgr->depool_lock);
 	isc_mutex_destroy(&mgr->buffer_lock);
 	isc_mutex_destroy(&mgr->lock);
 	isc_mem_putanddetach(&mctx, mgr, sizeof(dns_dispatchmgr_t));
@@ -1796,22 +1899,6 @@ dns_dispatchmgr_getblackhole(dns_dispatchmgr_t *mgr) {
 	return (mgr->blackhole);
 }
 
-void
-dns_dispatchmgr_setblackportlist(dns_dispatchmgr_t *mgr,
-				 dns_portlist_t *portlist) {
-	REQUIRE(VALID_DISPATCHMGR(mgr));
-	UNUSED(portlist);
-
-	/* This function is deprecated: use dns_dispatchmgr_setavailports(). */
-	return;
-}
-
-dns_portlist_t *
-dns_dispatchmgr_getblackportlist(dns_dispatchmgr_t *mgr) {
-	REQUIRE(VALID_DISPATCHMGR(mgr));
-	return (NULL); /* this function is deprecated */
-}
-
 isc_result_t
 dns_dispatchmgr_setavailports(dns_dispatchmgr_t *mgr, isc_portset_t *v4portset,
 			      isc_portset_t *v6portset) {
@@ -1877,7 +1964,6 @@ dns_dispatchmgr_setudp(dns_dispatchmgr_t *mgr, unsigned int buffersize,
 	REQUIRE(maxbuffers > 0);
 	REQUIRE(buckets < 2097169); /* next prime > 65536 * 32 */
 	REQUIRE(increment > buckets);
-	UNUSED(maxrequests);
 
 	/*
 	 * Keep some number of items around.  This should be a config
@@ -1898,15 +1984,48 @@ dns_dispatchmgr_setudp(dns_dispatchmgr_t *mgr, unsigned int buffersize,
 
 	LOCK(&mgr->buffer_lock);
 
-	if (maxbuffers > mgr->maxbuffers) {
-		mgr->maxbuffers = maxbuffers;
+	/* Create or adjust buffer pool */
+	if (mgr->bpool != NULL) {
+		/*
+		 * We only increase the maxbuffers to avoid accidental buffer
+		 * shortage.  Ideally we'd separate the manager-wide maximum
+		 * from per-dispatch limits and respect the latter within the
+		 * global limit.  But at this moment that's deemed to be
+		 * overkilling and isn't worth additional implementation
+		 * complexity.
+		 */
+		if (maxbuffers > mgr->maxbuffers) {
+			isc_mempool_setmaxalloc(mgr->bpool, maxbuffers);
+			isc_mempool_setfreemax(mgr->bpool, maxbuffers);
+			mgr->maxbuffers = maxbuffers;
+		}
+	} else {
+		isc_mempool_create(mgr->mctx, buffersize, &mgr->bpool);
+		isc_mempool_setname(mgr->bpool, "dispmgr_bpool");
+		isc_mempool_setmaxalloc(mgr->bpool, maxbuffers);
+		isc_mempool_setfreemax(mgr->bpool, maxbuffers);
+		isc_mempool_associatelock(mgr->bpool, &mgr->bpool_lock);
+		isc_mempool_setfillcount(mgr->bpool, 32);
 	}
 
 	/* Create or adjust socket pool */
-	if (mgr->qid != NULL) {
+	if (mgr->spool != NULL) {
+		if (maxrequests < DNS_DISPATCH_POOLSOCKS * 2) {
+			isc_mempool_setmaxalloc(mgr->spool,
+						DNS_DISPATCH_POOLSOCKS * 2);
+			isc_mempool_setfreemax(mgr->spool,
+					       DNS_DISPATCH_POOLSOCKS * 2);
+		}
 		UNLOCK(&mgr->buffer_lock);
 		return (ISC_R_SUCCESS);
 	}
+	isc_mempool_create(mgr->mctx, sizeof(dispsocket_t), &mgr->spool);
+
+	isc_mempool_setname(mgr->spool, "dispmgr_spool");
+	isc_mempool_setmaxalloc(mgr->spool, maxrequests);
+	isc_mempool_setfreemax(mgr->spool, maxrequests);
+	isc_mempool_associatelock(mgr->spool, &mgr->spool_lock);
+	isc_mempool_setfillcount(mgr->spool, 32);
 
 	result = qid_allocate(mgr, buckets, increment, &mgr->qid, true);
 	if (result != ISC_R_SUCCESS) {
@@ -1919,6 +2038,10 @@ dns_dispatchmgr_setudp(dns_dispatchmgr_t *mgr, unsigned int buffersize,
 	return (ISC_R_SUCCESS);
 
 cleanup:
+	isc_mempool_destroy(&mgr->bpool);
+	if (mgr->spool != NULL) {
+		isc_mempool_destroy(&mgr->spool);
+	}
 	UNLOCK(&mgr->buffer_lock);
 	return (result);
 }
@@ -2184,8 +2307,10 @@ dispatch_allocate(dns_dispatchmgr_t *mgr, unsigned int maxrequests,
 	 * the options that are controlled by tcp vs. udp, etc.
 	 */
 
-	disp = isc_mem_get(mgr->mctx, sizeof(*disp));
-	isc_refcount_increment0(&mgr->irefs);
+	disp = isc_mempool_get(mgr->dpool);
+	if (disp == NULL) {
+		return (ISC_R_NOMEMORY);
+	}
 
 	disp->magic = 0;
 	disp->mgr = mgr;
@@ -2209,6 +2334,7 @@ dispatch_allocate(dns_dispatchmgr_t *mgr, unsigned int maxrequests,
 	ISC_LIST_INIT(disp->inactivesockets);
 	disp->nsockets = 0;
 	disp->port_table = NULL;
+	disp->portpool = NULL;
 	disp->dscp = -1;
 
 	isc_mutex_init(&disp->lock);
@@ -2229,8 +2355,7 @@ dispatch_allocate(dns_dispatchmgr_t *mgr, unsigned int maxrequests,
 	 */
 kill_lock:
 	isc_mutex_destroy(&disp->lock);
-	isc_refcount_decrement(&mgr->irefs);
-	isc_mem_put(mgr->mctx, disp, sizeof(*disp));
+	isc_mempool_put(mgr->dpool, disp);
 
 	return (result);
 }
@@ -2261,8 +2386,7 @@ dispatch_free(dns_dispatch_t **dispp) {
 	INSIST(ISC_LIST_EMPTY(disp->activesockets));
 	INSIST(ISC_LIST_EMPTY(disp->inactivesockets));
 
-	isc_refcount_decrement(&mgr->irefs);
-	isc_mem_put(mgr->mctx, disp->failsafe_ev, sizeof(*disp->failsafe_ev));
+	isc_mempool_put(mgr->depool, disp->failsafe_ev);
 	disp->failsafe_ev = NULL;
 
 	if (disp->qid != NULL) {
@@ -2278,11 +2402,14 @@ dispatch_free(dns_dispatch_t **dispp) {
 				    DNS_DISPATCH_PORTTABLESIZE);
 	}
 
+	if (disp->portpool != NULL) {
+		isc_mempool_destroy(&disp->portpool);
+	}
+
 	disp->mgr = NULL;
 	isc_mutex_destroy(&disp->lock);
 	disp->magic = 0;
-	isc_refcount_decrement(&mgr->irefs);
-	isc_mem_put(mgr->mctx, disp, sizeof(*disp));
+	isc_mempool_put(mgr->dpool, disp);
 }
 
 isc_result_t
@@ -2758,6 +2885,11 @@ dispatch_createudp(dns_dispatchmgr_t *mgr, isc_socketmgr_t *sockmgr,
 		for (i = 0; i < DNS_DISPATCH_PORTTABLESIZE; i++) {
 			ISC_LIST_INIT(disp->port_table[i]);
 		}
+
+		isc_mempool_create(mgr->mctx, sizeof(dispportentry_t),
+				   &disp->portpool);
+		isc_mempool_setname(disp->portpool, "disp_portpool");
+		isc_mempool_setfreemax(disp->portpool, 128);
 	}
 	disp->socket = sock;
 	disp->local = *localaddr;
@@ -2785,8 +2917,15 @@ dispatch_createudp(dns_dispatchmgr_t *mgr, isc_socketmgr_t *sockmgr,
 				   destroy_disp, disp, sizeof(isc_event_t));
 
 	disp->sepool = NULL;
-	isc_mem_create(&disp->sepool);
-	isc_mem_setname(disp->sepool, "disp_sepool", NULL);
+	isc_mempool_create(mgr->mctx, sizeof(isc_socketevent_t), &disp->sepool);
+
+	isc_mutex_init(&disp->sepool_lock);
+
+	isc_mempool_setname(disp->sepool, "disp_sepool");
+	isc_mempool_setmaxalloc(disp->sepool, 32768);
+	isc_mempool_setfreemax(disp->sepool, 32768);
+	isc_mempool_associatelock(disp->sepool, &disp->sepool_lock);
+	isc_mempool_setfillcount(disp->sepool, 16);
 
 	attributes &= ~DNS_DISPATCHATTR_TCP;
 	attributes |= DNS_DISPATCHATTR_UDP;
@@ -3002,8 +3141,14 @@ dns_dispatch_addresponse(dns_dispatch_t *disp, unsigned int options,
 		return (ISC_R_NOMORE);
 	}
 
-	res = isc_mem_get(disp->mgr->mctx, sizeof(*res));
-	isc_refcount_increment0(&disp->mgr->irefs);
+	res = isc_mempool_get(disp->mgr->rpool);
+	if (res == NULL) {
+		if (dispsocket != NULL) {
+			destroy_dispsocket(disp, &dispsocket);
+		}
+		UNLOCK(&disp->lock);
+		return (ISC_R_NOMEMORY);
+	}
 
 	disp->refcount++;
 	disp->requests++;
@@ -3058,8 +3203,7 @@ dns_dispatch_addresponse(dns_dispatch_t *disp, unsigned int options,
 
 			UNLOCK(&disp->lock);
 			isc_task_detach(&res->task);
-			isc_refcount_decrement(&disp->mgr->irefs);
-			isc_mem_put(disp->mgr->mctx, res, sizeof(*res));
+			isc_mempool_put(disp->mgr->rpool, res);
 			return (result);
 		}
 	}
@@ -3247,8 +3391,7 @@ dns_dispatch_removeresponse(dns_dispentry_t **resp,
 		ev = ISC_LIST_HEAD(res->items);
 	}
 	res->magic = 0;
-	isc_refcount_decrement(&disp->mgr->irefs);
-	isc_mem_put(disp->mgr->mctx, res, sizeof(*res));
+	isc_mempool_put(disp->mgr->rpool, res);
 	if (disp->shutting_down == 1) {
 		do_cancel(disp);
 	} else {
