@@ -1,8 +1,6 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * SPDX-License-Identifier: MPL-2.0
- *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, you can obtain one at https://mozilla.org/MPL/2.0/.
@@ -20,7 +18,7 @@
 
 #define RRTYPE_A_ATTRIBUTES (0)
 
-static isc_result_t
+static inline isc_result_t
 fromtext_in_a(ARGS_FROMTEXT) {
 	isc_token_t token;
 	struct in_addr addr;
@@ -50,7 +48,7 @@ fromtext_in_a(ARGS_FROMTEXT) {
 	return (ISC_R_SUCCESS);
 }
 
-static isc_result_t
+static inline isc_result_t
 totext_in_a(ARGS_TOTEXT) {
 	isc_region_t region;
 
@@ -64,7 +62,7 @@ totext_in_a(ARGS_TOTEXT) {
 	return (inet_totext(AF_INET, tctx->flags, &region, target));
 }
 
-static isc_result_t
+static inline isc_result_t
 fromwire_in_a(ARGS_FROMWIRE) {
 	isc_region_t sregion;
 	isc_region_t tregion;
@@ -92,7 +90,7 @@ fromwire_in_a(ARGS_FROMWIRE) {
 	return (ISC_R_SUCCESS);
 }
 
-static isc_result_t
+static inline isc_result_t
 towire_in_a(ARGS_TOWIRE) {
 	isc_region_t region;
 
@@ -111,7 +109,7 @@ towire_in_a(ARGS_TOWIRE) {
 	return (ISC_R_SUCCESS);
 }
 
-static int
+static inline int
 compare_in_a(ARGS_COMPARE) {
 	isc_region_t r1;
 	isc_region_t r2;
@@ -128,7 +126,7 @@ compare_in_a(ARGS_COMPARE) {
 	return (isc_region_compare(&r1, &r2));
 }
 
-static isc_result_t
+static inline isc_result_t
 fromstruct_in_a(ARGS_FROMSTRUCT) {
 	dns_rdata_in_a_t *a = source;
 	uint32_t n;
@@ -147,7 +145,7 @@ fromstruct_in_a(ARGS_FROMSTRUCT) {
 	return (uint32_tobuffer(n, target));
 }
 
-static isc_result_t
+static inline isc_result_t
 tostruct_in_a(ARGS_TOSTRUCT) {
 	dns_rdata_in_a_t *a = target;
 	uint32_t n;
@@ -171,7 +169,7 @@ tostruct_in_a(ARGS_TOSTRUCT) {
 	return (ISC_R_SUCCESS);
 }
 
-static void
+static inline void
 freestruct_in_a(ARGS_FREESTRUCT) {
 	dns_rdata_in_a_t *a = source;
 
@@ -182,7 +180,7 @@ freestruct_in_a(ARGS_FREESTRUCT) {
 	UNUSED(a);
 }
 
-static isc_result_t
+static inline isc_result_t
 additionaldata_in_a(ARGS_ADDLDATA) {
 	REQUIRE(rdata->type == dns_rdatatype_a);
 	REQUIRE(rdata->rdclass == dns_rdataclass_in);
@@ -194,7 +192,7 @@ additionaldata_in_a(ARGS_ADDLDATA) {
 	return (ISC_R_SUCCESS);
 }
 
-static isc_result_t
+static inline isc_result_t
 digest_in_a(ARGS_DIGEST) {
 	isc_region_t r;
 
@@ -206,10 +204,9 @@ digest_in_a(ARGS_DIGEST) {
 	return ((digest)(arg, &r));
 }
 
-static bool
+static inline bool
 checkowner_in_a(ARGS_CHECKOWNER) {
 	dns_name_t prefix, suffix;
-	unsigned int labels, i;
 
 	REQUIRE(type == dns_rdatatype_a);
 	REQUIRE(rdclass == dns_rdataclass_in);
@@ -217,47 +214,24 @@ checkowner_in_a(ARGS_CHECKOWNER) {
 	UNUSED(type);
 	UNUSED(rdclass);
 
-	labels = dns_name_countlabels(name);
-	if (labels > 2U) {
-		/*
-		 * Handle Active Directory gc._msdcs.<forest> name.
-		 */
+	/*
+	 * Handle Active Directory gc._msdcs.<forest> name.
+	 */
+	if (dns_name_countlabels(name) > 2U) {
 		dns_name_init(&prefix, NULL);
 		dns_name_init(&suffix, NULL);
-		dns_name_split(name, labels - 2, &prefix, &suffix);
+		dns_name_split(name, dns_name_countlabels(name) - 2, &prefix,
+			       &suffix);
 		if (dns_name_equal(&gc_msdcs, &prefix) &&
 		    dns_name_ishostname(&suffix, false)) {
 			return (true);
 		}
-
-		/*
-		 * Handle SPF exists targets when the seperating label is:
-		 * - "_spf" RFC7208, section 5.7
-		 * - "_spf_verify" RFC7208, Appendix D1
-		 * - "_spf_rate" RFC7208, Appendix D1
-		 */
-		for (i = 0; i < labels - 2; i++) {
-			dns_label_t label;
-			dns_name_getlabel(name, i, &label);
-			if ((label.length == 5 &&
-			     strncasecmp((char *)label.base, "\x04_spf", 5) ==
-				     0) ||
-			    (label.length == 12 &&
-			     strncasecmp((char *)label.base, "\x0b_spf_verify",
-					 12) == 0) ||
-			    (label.length == 10 &&
-			     strncasecmp((char *)label.base, "\x09_spf_rate",
-					 10) == 0))
-			{
-				return (true);
-			}
-		}
 	}
 
 	return (dns_name_ishostname(name, wildcard));
 }
 
-static bool
+static inline bool
 checknames_in_a(ARGS_CHECKNAMES) {
 	REQUIRE(rdata->type == dns_rdatatype_a);
 	REQUIRE(rdata->rdclass == dns_rdataclass_in);
@@ -269,7 +243,7 @@ checknames_in_a(ARGS_CHECKNAMES) {
 	return (true);
 }
 
-static int
+static inline int
 casecompare_in_a(ARGS_COMPARE) {
 	return (compare_in_a(rdata1, rdata2));
 }
