@@ -75,34 +75,11 @@ set_mystdtime(int year, int month, int day) {
 	mystdtime = timegm(&tm);
 }
 
-/*
- * Override isc_stdtime_get() from lib/isc/[unix/win32]/stdtime.c
- * with our own for testing purposes.
- */
 void
 isc_stdtime_get(isc_stdtime_t *now) {
 	*now = mystdtime;
 }
 
-/*
- * Because update_test.o requires dns_update_*() symbols, the linker is able
- * to resolve them using libdns.a(update.o).  That object has other symbol
- * dependencies (dst_key_*()), so it pulls libdns.a(dst_api.o).
- * That object file requires the isc_stdtime_tostring() symbol.
- *
- * Define a local version here so that we don't have to depend on
- * libisc.a(stdtime.o).  If isc_stdtime_tostring() would be left undefined,
- * the linker has to get the required object file, and that will result in a
- * multiple definition error because the isc_stdtime_get() symbol exported
- * there is already in the exported list.
- */
-void
-isc_stdtime_tostring(isc_stdtime_t t, char *out, size_t outlen) {
-	UNUSED(t);
-	UNUSED(out);
-	UNUSED(outlen);
-}
-
 /* simple increment by 1 */
 static void
 increment_test(void **state) {
@@ -111,7 +88,7 @@ increment_test(void **state) {
 
 	UNUSED(state);
 
-	serial = dns_update_soaserial(old, dns_updatemethod_increment, NULL);
+	serial = dns_update_soaserial(old, dns_updatemethod_increment);
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, 51);
@@ -125,7 +102,7 @@ increment_past_zero_test(void **state) {
 
 	UNUSED(state);
 
-	serial = dns_update_soaserial(old, dns_updatemethod_increment, NULL);
+	serial = dns_update_soaserial(old, dns_updatemethod_increment);
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, 1u);
@@ -142,7 +119,7 @@ past_to_unix_test(void **state) {
 	set_mystdtime(2011, 6, 22);
 	old = mystdtime - 1;
 
-	serial = dns_update_soaserial(old, dns_updatemethod_unixtime, NULL);
+	serial = dns_update_soaserial(old, dns_updatemethod_unixtime);
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, mystdtime);
@@ -159,7 +136,7 @@ now_to_unix_test(void **state) {
 	set_mystdtime(2011, 6, 22);
 	old = mystdtime;
 
-	serial = dns_update_soaserial(old, dns_updatemethod_unixtime, NULL);
+	serial = dns_update_soaserial(old, dns_updatemethod_unixtime);
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, old + 1);
@@ -176,7 +153,7 @@ future_to_unix_test(void **state) {
 	set_mystdtime(2011, 6, 22);
 	old = mystdtime + 1;
 
-	serial = dns_update_soaserial(old, dns_updatemethod_unixtime, NULL);
+	serial = dns_update_soaserial(old, dns_updatemethod_unixtime);
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, old + 1);
@@ -194,7 +171,7 @@ undefined_plus1_to_unix_test(void **state) {
 	old = mystdtime ^ 0x80000000u;
 	old += 1;
 
-	serial = dns_update_soaserial(old, dns_updatemethod_unixtime, NULL);
+	serial = dns_update_soaserial(old, dns_updatemethod_unixtime);
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, mystdtime);
@@ -212,7 +189,7 @@ undefined_minus1_to_unix_test(void **state) {
 	old = mystdtime ^ 0x80000000u;
 	old -= 1;
 
-	serial = dns_update_soaserial(old, dns_updatemethod_unixtime, NULL);
+	serial = dns_update_soaserial(old, dns_updatemethod_unixtime);
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, old + 1);
@@ -229,7 +206,7 @@ undefined_to_unix_test(void **state) {
 	set_mystdtime(2011, 6, 22);
 	old = mystdtime ^ 0x80000000u;
 
-	serial = dns_update_soaserial(old, dns_updatemethod_unixtime, NULL);
+	serial = dns_update_soaserial(old, dns_updatemethod_unixtime);
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, old + 1);
@@ -246,7 +223,7 @@ unixtime_zero_test(void **state) {
 	mystdtime = 0;
 	old = 0xfffffff0;
 
-	serial = dns_update_soaserial(old, dns_updatemethod_unixtime, NULL);
+	serial = dns_update_soaserial(old, dns_updatemethod_unixtime);
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, old + 1);
@@ -256,19 +233,18 @@ unixtime_zero_test(void **state) {
 static void
 past_to_date_test(void **state) {
 	uint32_t old, serial;
-	dns_updatemethod_t used = dns_updatemethod_none;
 
 	UNUSED(state);
 
 	set_mystdtime(2014, 3, 31);
-	old = dns_update_soaserial(0, dns_updatemethod_date, NULL);
+	old = dns_update_soaserial(0, dns_updatemethod_date);
 	set_mystdtime(2014, 4, 1);
 
-	serial = dns_update_soaserial(old, dns_updatemethod_date, &used);
+	serial = dns_update_soaserial(old, dns_updatemethod_date);
+
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, 2014040100);
-	assert_int_equal(dns_updatemethod_date, used);
 }
 
 /* now to date */
@@ -276,35 +252,16 @@ static void
 now_to_date_test(void **state) {
 	uint32_t old;
 	uint32_t serial;
-	dns_updatemethod_t used = dns_updatemethod_none;
 
 	UNUSED(state);
 
 	set_mystdtime(2014, 4, 1);
-	old = dns_update_soaserial(0, dns_updatemethod_date, NULL);
+	old = dns_update_soaserial(0, dns_updatemethod_date);
 
-	serial = dns_update_soaserial(old, dns_updatemethod_date, &used);
+	serial = dns_update_soaserial(old, dns_updatemethod_date);
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, 2014040101);
-	assert_int_equal(dns_updatemethod_date, used);
-
-	old = 2014040198;
-	serial = dns_update_soaserial(old, dns_updatemethod_date, &used);
-	assert_true(isc_serial_lt(old, serial));
-	assert_int_not_equal(serial, 0);
-	assert_int_equal(serial, 2014040199);
-	assert_int_equal(dns_updatemethod_date, used);
-
-	/*
-	 * Stealing from "tomorrow".
-	 */
-	old = 2014040199;
-	serial = dns_update_soaserial(old, dns_updatemethod_date, &used);
-	assert_true(isc_serial_lt(old, serial));
-	assert_int_not_equal(serial, 0);
-	assert_int_equal(serial, 2014040200);
-	assert_int_equal(dns_updatemethod_increment, used);
 }
 
 /* future to date */
@@ -312,26 +269,17 @@ static void
 future_to_date_test(void **state) {
 	uint32_t old;
 	uint32_t serial;
-	dns_updatemethod_t used = dns_updatemethod_none;
 
 	UNUSED(state);
 
 	set_mystdtime(2014, 4, 1);
-	old = dns_update_soaserial(0, dns_updatemethod_date, NULL);
+	old = dns_update_soaserial(0, dns_updatemethod_date);
 	set_mystdtime(2014, 3, 31);
 
-	serial = dns_update_soaserial(old, dns_updatemethod_date, &used);
+	serial = dns_update_soaserial(old, dns_updatemethod_date);
 	assert_true(isc_serial_lt(old, serial));
 	assert_int_not_equal(serial, 0);
 	assert_int_equal(serial, 2014040101);
-	assert_int_equal(dns_updatemethod_increment, used);
-
-	old = serial;
-	serial = dns_update_soaserial(old, dns_updatemethod_date, &used);
-	assert_true(isc_serial_lt(old, serial));
-	assert_int_not_equal(serial, 0);
-	assert_int_equal(serial, 2014040102);
-	assert_int_equal(dns_updatemethod_increment, used);
 }
 
 int
@@ -373,7 +321,7 @@ main(void) {
 int
 main(void) {
 	printf("1..0 # Skipped: cmocka not available\n");
-	return (SKIPPED_TEST_EXIT_CODE);
+	return (0);
 }
 
 #endif /* if HAVE_CMOCKA */
