@@ -15,11 +15,10 @@
 #include <stdbool.h>
 
 #include <isc/mem.h>
+#include <isc/netmgr.h>
 #include <isc/print.h>
 #include <isc/random.h>
 #include <isc/string.h> /* Required for HP/UX (and others?) */
-#include <isc/task.h>
-#include <isc/timer.h>
 #include <isc/util.h>
 
 #include <dns/callbacks.h>
@@ -36,7 +35,6 @@
 #include <dns/result.h>
 #include <dns/soa.h>
 #include <dns/tcpmsg.h>
-#include <dns/timer.h>
 #include <dns/tsig.h>
 #include <dns/view.h>
 #include <dns/xfrin.h>
@@ -96,16 +94,16 @@ struct dns_xfrin_ctx {
 	isc_mem_t *mctx;
 	dns_zone_t *zone;
 
-	int refcount;
+	isc_refcount_t references;
 
-	isc_task_t *task;
-	isc_timer_t *timer;
-	isc_socketmgr_t *socketmgr;
+	isc_nm_t *netmgr;
+
+	isc_refcount_t connects; /*%< Connect in progress */
+	isc_refcount_t sends;	 /*%< Send in progress */
+	isc_refcount_t recvs;	 /*%< Receive in progress */
 
-	int connects; /*%< Connect in progress */
-	int sends;    /*%< Send in progress */
-	int recvs;    /*%< Receive in progress */
 	bool shuttingdown;
+
 	isc_result_t shutdown_result;
 
 	dns_name_t name; /*%< Name of zone to transfer */
@@ -123,19 +121,18 @@ struct dns_xfrin_ctx {
 
 	isc_sockaddr_t masteraddr;
 	isc_sockaddr_t sourceaddr;
-	isc_socket_t *socket;
+
+	isc_nmhandle_t *handle;
+	isc_nmhandle_t *readhandle;
+	isc_nmhandle_t *sendhandle;
 
 	/*% Buffer for IXFR/AXFR request message */
 	isc_buffer_t qbuffer;
 	unsigned char qbuffer_data[512];
 
-	/*% Incoming reply TCP message */
-	dns_tcpmsg_t tcpmsg;
-	bool tcpmsg_valid;
-
 	/*%
 	 * Whether the zone originally had a database attached at the time this
-	 * transfer context was created.  Used by maybe_free() when making
+	 * transfer context was created.  Used by xfrin_destroy() when making
 	 * logging decisions.
 	 */
 	bool zone_had_db;
@@ -188,9 +185,8 @@ struct dns_xfrin_ctx {
  * Forward declarations.
  */
 
-static isc_result_t
-xfrin_create(isc_mem_t *mctx, dns_zone_t *zone, dns_db_t *db, isc_task_t *task,
-	     isc_timermgr_t *timermgr, isc_socketmgr_t *socketmgr,
+static void
+xfrin_create(isc_mem_t *mctx, dns_zone_t *zone, dns_db_t *db, isc_nm_t *netmgr,
 	     dns_name_t *zonename, dns_rdataclass_t rdclass,
 	     dns_rdatatype_t reqtype, const isc_sockaddr_t *masteraddr,
 	     const isc_sockaddr_t *sourceaddr, isc_dscp_t dscp,
@@ -228,18 +224,17 @@ static isc_result_t
 xfrin_start(dns_xfrin_ctx_t *xfr);
 
 static void
-xfrin_connect_done(isc_task_t *task, isc_event_t *event);
+xfrin_connect_done(isc_nmhandle_t *handle, isc_result_t result, void *cbarg);
 static isc_result_t
 xfrin_send_request(dns_xfrin_ctx_t *xfr);
 static void
-xfrin_send_done(isc_task_t *task, isc_event_t *event);
+xfrin_send_done(isc_nmhandle_t *handle, isc_result_t result, void *cbarg);
 static void
-xfrin_recv_done(isc_task_t *task, isc_event_t *event);
-static void
-xfrin_timeout(isc_task_t *task, isc_event_t *event);
+xfrin_recv_done(isc_nmhandle_t *handle, isc_result_t result,
+		isc_region_t *region, void *cbarg);
 
 static void
-maybe_free(dns_xfrin_ctx_t *xfr);
+xfrin_destroy(dns_xfrin_ctx_t *xfr);
 
 static void
 xfrin_fail(dns_xfrin_ctx_t *xfr, isc_result_t result, const char *msg);
@@ -375,7 +370,7 @@ failure:
 static isc_result_t
 ixfr_init(dns_xfrin_ctx_t *xfr) {
 	isc_result_t result;
-	char *journalfile;
+	char *journalfile = NULL;
 
 	if (xfr->reqtype != dns_rdatatype_ixfr) {
 		xfrin_log(xfr, ISC_LOG_ERROR,
@@ -657,16 +652,15 @@ isc_result_t
 dns_xfrin_create(dns_zone_t *zone, dns_rdatatype_t xfrtype,
 		 const isc_sockaddr_t *masteraddr,
 		 const isc_sockaddr_t *sourceaddr, isc_dscp_t dscp,
-		 dns_tsigkey_t *tsigkey, isc_mem_t *mctx,
-		 isc_timermgr_t *timermgr, isc_socketmgr_t *socketmgr,
-		 isc_task_t *task, dns_xfrindone_t done,
-		 dns_xfrin_ctx_t **xfrp) {
+		 dns_tsigkey_t *tsigkey, isc_mem_t *mctx, isc_nm_t *netmgr,
+		 dns_xfrindone_t done, dns_xfrin_ctx_t **xfrp) {
 	dns_name_t *zonename = dns_zone_getorigin(zone);
 	dns_xfrin_ctx_t *xfr = NULL;
 	isc_result_t result;
 	dns_db_t *db = NULL;
 
 	REQUIRE(xfrp != NULL && *xfrp == NULL);
+	REQUIRE(done != NULL);
 
 	(void)dns_zone_getdb(zone, &db);
 
@@ -674,37 +668,50 @@ dns_xfrin_create(dns_zone_t *zone, dns_rdatatype_t xfrtype,
 		REQUIRE(db != NULL);
 	}
 
-	CHECK(xfrin_create(mctx, zone, db, task, timermgr, socketmgr, zonename,
-			   dns_zone_getclass(zone), xfrtype, masteraddr,
-			   sourceaddr, dscp, tsigkey, &xfr));
+	xfrin_create(mctx, zone, db, netmgr, zonename, dns_zone_getclass(zone),
+		     xfrtype, masteraddr, sourceaddr, dscp, tsigkey, &xfr);
 
 	if (db != NULL) {
 		xfr->zone_had_db = true;
 	}
 
-	CHECK(xfrin_start(xfr));
-
 	xfr->done = done;
-	if (xfr->done != NULL) {
-		xfr->refcount++;
-	}
+
+	isc_refcount_init(&xfr->references, 1);
+
+	/*
+	 * Set *xfrp now, before calling xfrin_start(). Asynchronous
+	 * netmgr processing could cause the 'done' callback to run in
+	 * another thread before we reached the end of the present
+	 * function. In that case, if *xfrp hadn't already been
+	 * attached, the 'done' function would be unable to detach it.
+	 */
 	*xfrp = xfr;
 
-failure:
+	result = xfrin_start(xfr);
+	if (result != ISC_R_SUCCESS) {
+		xfr->shuttingdown = true;
+		xfr->shutdown_result = result;
+		dns_xfrin_detach(xfrp);
+	}
+
 	if (db != NULL) {
 		dns_db_detach(&db);
 	}
+
 	if (result != ISC_R_SUCCESS) {
 		char zonetext[DNS_NAME_MAXTEXT + 32];
 		dns_zone_name(zone, zonetext, sizeof(zonetext));
 		xfrin_log1(ISC_LOG_ERROR, zonetext, masteraddr,
 			   "zone transfer setup failed");
 	}
+
 	return (result);
 }
 
 void
 dns_xfrin_shutdown(dns_xfrin_ctx_t *xfr) {
+	REQUIRE(VALID_XFRIN(xfr));
 	if (!xfr->shuttingdown) {
 		xfrin_fail(xfr, ISC_R_CANCELED, "shut down");
 	}
@@ -712,30 +719,35 @@ dns_xfrin_shutdown(dns_xfrin_ctx_t *xfr) {
 
 void
 dns_xfrin_attach(dns_xfrin_ctx_t *source, dns_xfrin_ctx_t **target) {
+	REQUIRE(VALID_XFRIN(source));
 	REQUIRE(target != NULL && *target == NULL);
-	source->refcount++;
+	(void)isc_refcount_increment(&source->references);
+
 	*target = source;
 }
 
 void
 dns_xfrin_detach(dns_xfrin_ctx_t **xfrp) {
-	dns_xfrin_ctx_t *xfr = *xfrp;
+	dns_xfrin_ctx_t *xfr = NULL;
+
+	REQUIRE(xfrp != NULL && VALID_XFRIN(*xfrp));
+
+	xfr = *xfrp;
 	*xfrp = NULL;
-	INSIST(xfr->refcount > 0);
-	xfr->refcount--;
-	maybe_free(xfr);
+
+	if (isc_refcount_decrement(&xfr->references) == 1) {
+		xfrin_destroy(xfr);
+	}
 }
 
 static void
 xfrin_cancelio(dns_xfrin_ctx_t *xfr) {
-	if (xfr->connects > 0) {
-		isc_socket_cancel(xfr->socket, xfr->task,
-				  ISC_SOCKCANCEL_CONNECT);
-	} else if (xfr->recvs > 0) {
-		dns_tcpmsg_cancelread(&xfr->tcpmsg);
-	} else if (xfr->sends > 0) {
-		isc_socket_cancel(xfr->socket, xfr->task, ISC_SOCKCANCEL_SEND);
+	if (xfr->readhandle == NULL) {
+		return;
 	}
+
+	isc_nm_cancelread(xfr->readhandle);
+	/* The xfr->readhandle detach will happen in xfrin_recv_done callback */
 }
 
 static void
@@ -744,11 +756,8 @@ xfrin_reset(dns_xfrin_ctx_t *xfr) {
 
 	xfrin_log(xfr, ISC_LOG_INFO, "resetting");
 
-	xfrin_cancelio(xfr);
-
-	if (xfr->socket != NULL) {
-		isc_socket_detach(&xfr->socket);
-	}
+	REQUIRE(xfr->readhandle == NULL);
+	REQUIRE(xfr->sendhandle == NULL);
 
 	if (xfr->lasttsig != NULL) {
 		isc_buffer_free(&xfr->lasttsig);
@@ -765,11 +774,6 @@ xfrin_reset(dns_xfrin_ctx_t *xfr) {
 		(void)dns_db_endload(xfr->db, &xfr->axfr);
 	}
 
-	if (xfr->tcpmsg_valid) {
-		dns_tcpmsg_invalidate(&xfr->tcpmsg);
-		xfr->tcpmsg_valid = false;
-	}
-
 	if (xfr->ver != NULL) {
 		dns_db_closeversion(xfr->db, &xfr->ver, false);
 	}
@@ -798,99 +802,52 @@ xfrin_fail(dns_xfrin_ctx_t *xfr, isc_result_t result, const char *msg) {
 	}
 	xfr->shuttingdown = true;
 	xfr->shutdown_result = result;
-	maybe_free(xfr);
 }
 
-static isc_result_t
-xfrin_create(isc_mem_t *mctx, dns_zone_t *zone, dns_db_t *db, isc_task_t *task,
-	     isc_timermgr_t *timermgr, isc_socketmgr_t *socketmgr,
+static void
+xfrin_create(isc_mem_t *mctx, dns_zone_t *zone, dns_db_t *db, isc_nm_t *netmgr,
 	     dns_name_t *zonename, dns_rdataclass_t rdclass,
 	     dns_rdatatype_t reqtype, const isc_sockaddr_t *masteraddr,
 	     const isc_sockaddr_t *sourceaddr, isc_dscp_t dscp,
 	     dns_tsigkey_t *tsigkey, dns_xfrin_ctx_t **xfrp) {
 	dns_xfrin_ctx_t *xfr = NULL;
-	isc_result_t result;
 
 	xfr = isc_mem_get(mctx, sizeof(*xfr));
-	xfr->mctx = NULL;
+	*xfr = (dns_xfrin_ctx_t){ .netmgr = netmgr,
+				  .shutdown_result = ISC_R_UNSET,
+				  .rdclass = rdclass,
+				  .reqtype = reqtype,
+				  .dscp = dscp,
+				  .id = (dns_messageid_t)isc_random16(),
+				  .maxrecords = dns_zone_getmaxrecords(zone),
+				  .masteraddr = *masteraddr,
+				  .sourceaddr = *sourceaddr };
+
 	isc_mem_attach(mctx, &xfr->mctx);
-	xfr->refcount = 0;
-	xfr->zone = NULL;
 	dns_zone_iattach(zone, &xfr->zone);
-	xfr->task = NULL;
-	isc_task_attach(task, &xfr->task);
-	xfr->timer = NULL;
-	xfr->socketmgr = socketmgr;
-	xfr->done = NULL;
-
-	xfr->connects = 0;
-	xfr->sends = 0;
-	xfr->recvs = 0;
-	xfr->shuttingdown = false;
-	xfr->shutdown_result = ISC_R_UNSET;
-
 	dns_name_init(&xfr->name, NULL);
-	xfr->rdclass = rdclass;
-	xfr->id = (dns_messageid_t)isc_random16();
-	xfr->reqtype = reqtype;
-	xfr->dscp = dscp;
 
-	/* sockaddr */
-	xfr->socket = NULL;
-	/* qbuffer */
-	/* qbuffer_data */
-	/* tcpmsg */
-	xfr->tcpmsg_valid = false;
-
-	xfr->zone_had_db = false;
-	xfr->db = NULL;
 	if (db != NULL) {
 		dns_db_attach(db, &xfr->db);
 	}
-	xfr->ver = NULL;
+
 	dns_diff_init(xfr->mctx, &xfr->diff);
-	xfr->difflen = 0;
 
 	if (reqtype == dns_rdatatype_soa) {
 		xfr->state = XFRST_SOAQUERY;
 	} else {
 		xfr->state = XFRST_INITIALSOA;
 	}
-	/* end_serial */
 
-	xfr->nmsg = 0;
-	xfr->nrecs = 0;
-	xfr->nbytes = 0;
-	xfr->maxrecords = dns_zone_getmaxrecords(zone);
 	isc_time_now(&xfr->start);
 
-	xfr->tsigkey = NULL;
 	if (tsigkey != NULL) {
 		dns_tsigkey_attach(tsigkey, &xfr->tsigkey);
 	}
-	xfr->lasttsig = NULL;
-	xfr->tsigctx = NULL;
-	xfr->sincetsig = 0;
-	xfr->is_ixfr = false;
-
-	/* ixfr.request_serial */
-	/* ixfr.current_serial */
-	xfr->ixfr.journal = NULL;
-
-	xfr->axfr.add = NULL;
-	xfr->axfr.add_private = NULL;
 
 	dns_name_dup(zonename, mctx, &xfr->name);
 
-	CHECK(isc_timer_create(timermgr, isc_timertype_inactive, NULL, NULL,
-			       task, xfrin_timeout, xfr, &xfr->timer));
-	CHECK(dns_timer_setidle(xfr->timer, dns_zone_getmaxxfrin(xfr->zone),
-				dns_zone_getidlein(xfr->zone), false));
-
-	xfr->masteraddr = *masteraddr;
-
 	INSIST(isc_sockaddr_pf(masteraddr) == isc_sockaddr_pf(sourceaddr));
-	xfr->sourceaddr = *sourceaddr;
 	isc_sockaddr_setport(&xfr->sourceaddr, 0);
 
 	/*
@@ -900,47 +857,30 @@ xfrin_create(isc_mem_t *mctx, dns_zone_t *zone, dns_db_t *db, isc_task_t *task,
 			sizeof(xfr->qbuffer_data) - 2);
 
 	xfr->magic = XFRIN_MAGIC;
+
 	*xfrp = xfr;
-	return (ISC_R_SUCCESS);
-
-failure:
-	if (xfr->timer != NULL) {
-		isc_timer_detach(&xfr->timer);
-	}
-	if (dns_name_dynamic(&xfr->name)) {
-		dns_name_free(&xfr->name, xfr->mctx);
-	}
-	if (xfr->tsigkey != NULL) {
-		dns_tsigkey_detach(&xfr->tsigkey);
-	}
-	if (xfr->db != NULL) {
-		dns_db_detach(&xfr->db);
-	}
-	isc_task_detach(&xfr->task);
-	dns_zone_idetach(&xfr->zone);
-	isc_mem_putanddetach(&xfr->mctx, xfr, sizeof(*xfr));
-
-	return (result);
 }
 
 static isc_result_t
 xfrin_start(dns_xfrin_ctx_t *xfr) {
 	isc_result_t result;
-	CHECK(isc_socket_create(xfr->socketmgr,
-				isc_sockaddr_pf(&xfr->sourceaddr),
-				isc_sockettype_tcp, &xfr->socket));
-	isc_socket_setname(xfr->socket, "xfrin", NULL);
-#ifndef BROKEN_TCP_BIND_BEFORE_CONNECT
-	CHECK(isc_socket_bind(xfr->socket, &xfr->sourceaddr,
-			      ISC_SOCKET_REUSEADDRESS));
-#endif /* ifndef BROKEN_TCP_BIND_BEFORE_CONNECT */
-	isc_socket_dscp(xfr->socket, xfr->dscp);
-	CHECK(isc_socket_connect(xfr->socket, &xfr->masteraddr, xfr->task,
-				 xfrin_connect_done, xfr));
-	xfr->connects++;
+	dns_xfrin_ctx_t *connect_xfr = NULL;
+	/*
+	 * XXX: timeout hard-coded to 30 seconds; this needs to be
+	 * configurable.
+	 */
+	(void)isc_refcount_increment0(&xfr->connects);
+	dns_xfrin_attach(xfr, &connect_xfr);
+	CHECK(isc_nm_tcpdnsconnect(xfr->netmgr,
+				   (isc_nmiface_t *)&xfr->sourceaddr,
+				   (isc_nmiface_t *)&xfr->masteraddr,
+				   xfrin_connect_done, connect_xfr, 30000, 0));
+	/* TODO	isc_socket_dscp(xfr->socket, xfr->dscp); */
 	return (ISC_R_SUCCESS);
+
 failure:
-	xfrin_fail(xfr, result, "failed setting up socket");
+	isc_refcount_decrement0(&xfr->connects);
+	dns_xfrin_detach(&connect_xfr);
 	return (result);
 }
 
@@ -972,49 +912,41 @@ failure:
  * A connection has been established.
  */
 static void
-xfrin_connect_done(isc_task_t *task, isc_event_t *event) {
-	isc_socket_connev_t *cev = (isc_socket_connev_t *)event;
-	dns_xfrin_ctx_t *xfr = (dns_xfrin_ctx_t *)event->ev_arg;
-	isc_result_t result = cev->result;
+xfrin_connect_done(isc_nmhandle_t *handle, isc_result_t result, void *cbarg) {
+	dns_xfrin_ctx_t *xfr = (dns_xfrin_ctx_t *)cbarg;
 	char sourcetext[ISC_SOCKADDR_FORMATSIZE];
 	char signerbuf[DNS_NAME_FORMATSIZE];
 	const char *signer = "", *sep = "";
 	isc_sockaddr_t sockaddr;
-	dns_zonemgr_t *zmgr;
+	dns_zonemgr_t *zmgr = NULL;
 	isc_time_t now;
 
 	REQUIRE(VALID_XFRIN(xfr));
 
-	UNUSED(task);
+	isc_refcount_decrement0(&xfr->connects);
 
-	INSIST(event->ev_type == ISC_SOCKEVENT_CONNECT);
-	isc_event_free(&event);
-
-	xfr->connects--;
 	if (xfr->shuttingdown) {
-		maybe_free(xfr);
-		return;
+		result = ISC_R_SHUTTINGDOWN;
 	}
 
+	CHECK(result);
+
 	zmgr = dns_zone_getmgr(xfr->zone);
 	if (zmgr != NULL) {
 		if (result != ISC_R_SUCCESS) {
 			TIME_NOW(&now);
 			dns_zonemgr_unreachableadd(zmgr, &xfr->masteraddr,
 						   &xfr->sourceaddr, &now);
-			goto failure;
+			CHECK(result);
 		} else {
 			dns_zonemgr_unreachabledel(zmgr, &xfr->masteraddr,
 						   &xfr->sourceaddr);
 		}
 	}
 
-	result = isc_socket_getsockname(xfr->socket, &sockaddr);
-	if (result == ISC_R_SUCCESS) {
-		isc_sockaddr_format(&sockaddr, sourcetext, sizeof(sourcetext));
-	} else {
-		strlcpy(sourcetext, "<UNKNOWN>", sizeof(sourcetext));
-	}
+	xfr->handle = handle;
+	sockaddr = isc_nmhandle_peeraddr(handle);
+	isc_sockaddr_format(&sockaddr, sourcetext, sizeof(sourcetext));
 
 	if (xfr->tsigkey != NULL && xfr->tsigkey->key != NULL) {
 		dns_name_format(dst_key_name(xfr->tsigkey->key), signerbuf,
@@ -1026,14 +958,16 @@ xfrin_connect_done(isc_task_t *task, isc_event_t *event) {
 	xfrin_log(xfr, ISC_LOG_INFO, "connected using %s%s%s", sourcetext, sep,
 		  signer);
 
-	dns_tcpmsg_init(xfr->mctx, xfr->socket, &xfr->tcpmsg);
-	xfr->tcpmsg_valid = true;
-
 	CHECK(xfrin_send_request(xfr));
+
 failure:
-	if (result != ISC_R_SUCCESS) {
+	isc_nmhandle_detach(&handle);
+
+	if (result != ISC_R_SUCCESS && result != ISC_R_SHUTTINGDOWN) {
 		xfrin_fail(xfr, result, "failed to connect");
 	}
+
+	dns_xfrin_detach(&xfr); /* connect_xfr */
 }
 
 /*
@@ -1102,6 +1036,7 @@ xfrin_send_request(dns_xfrin_ctx_t *xfr) {
 	dns_name_t *qname = NULL;
 	dns_dbversion_t *ver = NULL;
 	dns_name_t *msgsoaname = NULL;
+	dns_xfrin_ctx_t *send_xfr = NULL;
 
 	/* Create the request message */
 	dns_message_create(xfr->mctx, DNS_MESSAGE_INTENTRENDER, &msg);
@@ -1167,17 +1102,10 @@ xfrin_send_request(dns_xfrin_ctx_t *xfr) {
 	isc_buffer_usedregion(&xfr->qbuffer, &region);
 	INSIST(region.length <= 65535);
 
-	/*
-	 * Record message length and adjust region to include TCP
-	 * length field.
-	 */
-	xfr->qbuffer_data[0] = (region.length >> 8) & 0xff;
-	xfr->qbuffer_data[1] = region.length & 0xff;
-	region.base -= 2;
-	region.length += 2;
-	CHECK(isc_socket_send(xfr->socket, &region, xfr->task, xfrin_send_done,
-			      xfr));
-	xfr->sends++;
+	dns_xfrin_attach(xfr, &send_xfr);
+	isc_nmhandle_attach(send_xfr->handle, &xfr->sendhandle);
+	isc_refcount_increment0(&send_xfr->sends);
+	isc_nm_send(xfr->handle, &region, xfrin_send_done, send_xfr);
 
 failure:
 	if (qname != NULL) {
@@ -1195,64 +1123,61 @@ failure:
 	if (ver != NULL) {
 		dns_db_closeversion(xfr->db, &ver, false);
 	}
+
 	return (result);
 }
 
 static void
-xfrin_send_done(isc_task_t *task, isc_event_t *event) {
-	isc_socketevent_t *sev = (isc_socketevent_t *)event;
-	dns_xfrin_ctx_t *xfr = (dns_xfrin_ctx_t *)event->ev_arg;
-	isc_result_t result;
+xfrin_send_done(isc_nmhandle_t *handle, isc_result_t result, void *cbarg) {
+	dns_xfrin_ctx_t *xfr = (dns_xfrin_ctx_t *)cbarg;
+	dns_xfrin_ctx_t *recv_xfr = NULL;
 
 	REQUIRE(VALID_XFRIN(xfr));
 
-	UNUSED(task);
+	isc_refcount_decrement0(&xfr->sends);
+	if (xfr->shuttingdown) {
+		result = ISC_R_SHUTTINGDOWN;
+	}
 
-	INSIST(event->ev_type == ISC_SOCKEVENT_SENDDONE);
+	CHECK(result);
 
-	xfr->sends--;
 	xfrin_log(xfr, ISC_LOG_DEBUG(3), "sent request data");
-	CHECK(sev->result);
 
-	CHECK(dns_tcpmsg_readmessage(&xfr->tcpmsg, xfr->task, xfrin_recv_done,
-				     xfr));
-	xfr->recvs++;
+	dns_xfrin_attach(xfr, &recv_xfr);
+	isc_nmhandle_attach(handle, &recv_xfr->readhandle);
+	isc_refcount_increment0(&recv_xfr->recvs);
+	isc_nm_read(recv_xfr->handle, xfrin_recv_done, recv_xfr);
+
 failure:
-	isc_event_free(&event);
-	if (result != ISC_R_SUCCESS) {
+	if (result != ISC_R_SUCCESS && result != ISC_R_SHUTTINGDOWN) {
 		xfrin_fail(xfr, result, "failed sending request data");
 	}
+
+	isc_nmhandle_detach(&xfr->sendhandle);
+	dns_xfrin_detach(&xfr); /* send_xfr */
 }
 
 static void
-xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
-	dns_xfrin_ctx_t *xfr = (dns_xfrin_ctx_t *)ev->ev_arg;
-	isc_result_t result;
+xfrin_recv_done(isc_nmhandle_t *handle, isc_result_t result,
+		isc_region_t *region, void *cbarg) {
+	dns_xfrin_ctx_t *xfr = (dns_xfrin_ctx_t *)cbarg;
 	dns_message_t *msg = NULL;
-	dns_name_t *name;
-	dns_tcpmsg_t *tcpmsg;
+	dns_name_t *name = NULL;
 	const dns_name_t *tsigowner = NULL;
+	isc_buffer_t buffer;
+	isc_sockaddr_t peer;
 
 	REQUIRE(VALID_XFRIN(xfr));
 
-	UNUSED(task);
+	isc_refcount_decrement0(&xfr->recvs);
 
-	INSIST(ev->ev_type == DNS_EVENT_TCPMSG);
-	tcpmsg = ev->ev_sender;
-	isc_event_free(&ev);
-
-	xfr->recvs--;
 	if (xfr->shuttingdown) {
-		maybe_free(xfr);
-		return;
+		result = ISC_R_SHUTTINGDOWN;
 	}
 
-	CHECK(tcpmsg->result);
+	CHECK(result);
 
-	xfrin_log(xfr, ISC_LOG_DEBUG(7), "received %u bytes",
-		  tcpmsg->buffer.used);
-
-	CHECK(isc_timer_touch(xfr->timer));
+	xfrin_log(xfr, ISC_LOG_DEBUG(7), "received %u bytes", region->length);
 
 	dns_message_create(xfr->mctx, DNS_MESSAGE_INTENTPARSE, &msg);
 
@@ -1268,12 +1193,15 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 		msg->tcp_continuation = 1;
 	}
 
-	result = dns_message_parse(msg, &tcpmsg->buffer,
-				   DNS_MESSAGEPARSE_PRESERVEORDER);
+	isc_buffer_init(&buffer, region->base, region->length);
+	isc_buffer_add(&buffer, region->length);
+	peer = isc_nmhandle_peeraddr(handle);
 
+	result = dns_message_parse(msg, &buffer,
+				   DNS_MESSAGEPARSE_PRESERVEORDER);
 	if (result == ISC_R_SUCCESS) {
-		dns_message_logpacket(msg, "received message from",
-				      &tcpmsg->address, DNS_LOGCATEGORY_XFER_IN,
+		dns_message_logpacket(msg, "received message from", &peer,
+				      DNS_LOGCATEGORY_XFER_IN,
 				      DNS_LOGMODULE_XFER_IN, ISC_LOG_DEBUG(10),
 				      xfr->mctx);
 	} else {
@@ -1297,18 +1225,25 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 		} else if (result == ISC_R_SUCCESS || result == DNS_R_NOERROR) {
 			result = DNS_R_UNEXPECTEDID;
 		}
+
 		if (xfr->reqtype == dns_rdatatype_axfr ||
 		    xfr->reqtype == dns_rdatatype_soa) {
 			goto failure;
 		}
+
 		xfrin_log(xfr, ISC_LOG_DEBUG(3), "got %s, retrying with AXFR",
 			  isc_result_totext(result));
 	try_axfr:
+		isc_nmhandle_detach(&xfr->readhandle);
 		dns_message_detach(&msg);
 		xfrin_reset(xfr);
 		xfr->reqtype = dns_rdatatype_soa;
 		xfr->state = XFRST_SOAQUERY;
-		(void)xfrin_start(xfr);
+		result = xfrin_start(xfr);
+		if (result != ISC_R_SUCCESS) {
+			xfrin_fail(xfr, result, "failed setting up socket");
+		}
+		dns_xfrin_detach(&xfr); /* recv_xfr */
 		return;
 	}
 
@@ -1338,7 +1273,7 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 	     result == ISC_R_SUCCESS;
 	     result = dns_message_nextname(msg, DNS_SECTION_QUESTION))
 	{
-		dns_rdataset_t *rds;
+		dns_rdataset_t *rds = NULL;
 
 		name = NULL;
 		dns_message_currentname(msg, DNS_SECTION_QUESTION, &name);
@@ -1398,7 +1333,7 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 	     result == ISC_R_SUCCESS;
 	     result = dns_message_nextname(msg, DNS_SECTION_ANSWER))
 	{
-		dns_rdataset_t *rds;
+		dns_rdataset_t *rds = NULL;
 
 		name = NULL;
 		dns_message_currentname(msg, DNS_SECTION_ANSWER, &name);
@@ -1455,7 +1390,7 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 	/*
 	 * Update the number of bytes received.
 	 */
-	xfr->nbytes += tcpmsg->buffer.used;
+	xfr->nbytes += buffer.used;
 
 	/*
 	 * Take the context back.
@@ -1464,8 +1399,6 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 	xfr->tsigctx = msg->tsigctx;
 	msg->tsigctx = NULL;
 
-	dns_message_detach(&msg);
-
 	switch (xfr->state) {
 	case XFRST_GOTSOA:
 		xfr->reqtype = dns_rdatatype_axfr;
@@ -1474,7 +1407,7 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 		break;
 	case XFRST_AXFR_END:
 		CHECK(axfr_finalize(xfr));
-	/* FALLTHROUGH */
+		/* FALLTHROUGH */
 	case XFRST_IXFR_END:
 		/*
 		 * Close the journal.
@@ -1490,71 +1423,57 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 			(xfr->done)(xfr->zone, ISC_R_SUCCESS);
 			xfr->done = NULL;
 		}
-		/*
-		 * We should have no outstanding events at this
-		 * point, thus maybe_free() should succeed.
-		 */
+
 		xfr->shuttingdown = true;
 		xfr->shutdown_result = ISC_R_SUCCESS;
-		maybe_free(xfr);
 		break;
 	default:
 		/*
 		 * Read the next message.
 		 */
-		CHECK(dns_tcpmsg_readmessage(&xfr->tcpmsg, xfr->task,
-					     xfrin_recv_done, xfr));
-		xfr->recvs++;
+		/* The readhandle is still attached */
+		/* The recv_xfr is still attached */
+		dns_message_detach(&msg);
+		isc_refcount_increment0(&xfr->recvs);
+		isc_nm_read(xfr->handle, xfrin_recv_done, xfr);
+		return;
 	}
-	return;
 
 failure:
+	if (result != ISC_R_SUCCESS && result != ISC_R_SHUTTINGDOWN) {
+		xfrin_fail(xfr, result, "failed while receiving responses");
+	}
+
 	if (msg != NULL) {
 		dns_message_detach(&msg);
 	}
-	if (result != ISC_R_SUCCESS) {
-		xfrin_fail(xfr, result, "failed while receiving responses");
-	}
+	isc_nmhandle_detach(&xfr->readhandle);
+	dns_xfrin_detach(&xfr); /* recv_xfr */
 }
 
 static void
-xfrin_timeout(isc_task_t *task, isc_event_t *event) {
-	dns_xfrin_ctx_t *xfr = (dns_xfrin_ctx_t *)event->ev_arg;
-
-	REQUIRE(VALID_XFRIN(xfr));
-
-	UNUSED(task);
-
-	isc_event_free(&event);
-	/*
-	 * This will log "giving up: timeout".
-	 */
-	xfrin_fail(xfr, ISC_R_TIMEDOUT, "giving up");
-}
-
-static void
-maybe_free(dns_xfrin_ctx_t *xfr) {
+xfrin_destroy(dns_xfrin_ctx_t *xfr) {
 	uint64_t msecs;
 	uint64_t persec;
 	const char *result_str;
 
 	REQUIRE(VALID_XFRIN(xfr));
 
-	if (!xfr->shuttingdown || xfr->refcount != 0 || xfr->connects != 0 ||
-	    xfr->sends != 0 || xfr->recvs != 0)
-	{
-		return;
-	}
+	/* Safe-guards */
+	REQUIRE(xfr->shuttingdown);
+	isc_refcount_destroy(&xfr->references);
+	isc_refcount_destroy(&xfr->connects);
+	isc_refcount_destroy(&xfr->recvs);
+	isc_refcount_destroy(&xfr->sends);
 
-	INSIST(!xfr->shuttingdown || xfr->shutdown_result != ISC_R_UNSET);
+	INSIST(xfr->shutdown_result != ISC_R_UNSET);
 
-	/* If we're called through dns_xfrin_detach() and are not
+	/*
+	 * If we're called through dns_xfrin_detach() and are not
 	 * shutting down, we can't know what the transfer status is as
 	 * we are only called when the last reference is lost.
 	 */
-	result_str = (xfr->shuttingdown
-			      ? isc_result_totext(xfr->shutdown_result)
-			      : "unknown");
+	result_str = isc_result_totext(xfr->shutdown_result);
 	xfrin_log(xfr, ISC_LOG_INFO, "Transfer status: %s", result_str);
 
 	/*
@@ -1575,16 +1494,11 @@ maybe_free(dns_xfrin_ctx_t *xfr) {
 		  (unsigned int)(msecs / 1000), (unsigned int)(msecs % 1000),
 		  (unsigned int)persec, xfr->end_serial);
 
-	if (xfr->socket != NULL) {
-		isc_socket_detach(&xfr->socket);
+	if (xfr->readhandle != NULL) {
+		isc_nmhandle_detach(&xfr->readhandle);
 	}
-
-	if (xfr->timer != NULL) {
-		isc_timer_detach(&xfr->timer);
-	}
-
-	if (xfr->task != NULL) {
-		isc_task_detach(&xfr->task);
+	if (xfr->sendhandle != NULL) {
+		isc_nmhandle_detach(&xfr->sendhandle);
 	}
 
 	if (xfr->tsigkey != NULL) {
@@ -1605,10 +1519,6 @@ maybe_free(dns_xfrin_ctx_t *xfr) {
 		(void)dns_db_endload(xfr->db, &xfr->axfr);
 	}
 
-	if (xfr->tcpmsg_valid) {
-		dns_tcpmsg_invalidate(&xfr->tcpmsg);
-	}
-
 	if (xfr->tsigctx != NULL) {
 		dst_context_destroy(&xfr->tsigctx);
 	}
@@ -1626,7 +1536,7 @@ maybe_free(dns_xfrin_ctx_t *xfr) {
 	}
 
 	if (xfr->zone != NULL) {
-		if (!xfr->zone_had_db && xfr->shuttingdown &&
+		if (!xfr->zone_had_db &&
 		    xfr->shutdown_result == ISC_R_SUCCESS &&
 		    dns_zone_gettype(xfr->zone) == dns_zone_mirror)
 		{
