@@ -1,8 +1,6 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * SPDX-License-Identifier: MPL-2.0
- *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, you can obtain one at https://mozilla.org/MPL/2.0/.
@@ -97,15 +95,6 @@ struct nsec3_chain_fixed {
 	 */
 };
 
-/*
- * Helper function used to calculate length of variable-length
- * data section in object pointed to by 'chain'.
- */
-static size_t
-chain_length(struct nsec3_chain_fixed *chain) {
-	return (chain->salt_length + 2 * chain->next_length);
-}
-
 /*%
  * Log a zone verification error described by 'fmt' and the variable arguments
  * following it.  Either use dns_zone_logv() or print to stderr, depending on
@@ -172,22 +161,37 @@ has_dname(const vctx_t *vctx, dns_dbnode_t *node) {
 
 static bool
 goodsig(const vctx_t *vctx, dns_rdata_t *sigrdata, const dns_name_t *name,
-	dst_key_t **dstkeys, size_t nkeys, dns_rdataset_t *rdataset) {
+	dns_rdataset_t *keyrdataset, dns_rdataset_t *rdataset) {
+	dns_rdata_dnskey_t key;
 	dns_rdata_rrsig_t sig;
+	dst_key_t *dstkey = NULL;
 	isc_result_t result;
 
 	result = dns_rdata_tostruct(sigrdata, &sig, NULL);
 	RUNTIME_CHECK(result == ISC_R_SUCCESS);
 
-	for (size_t key = 0; key < nkeys; key++) {
-		if (sig.algorithm != dst_key_alg(dstkeys[key]) ||
-		    sig.keyid != dst_key_id(dstkeys[key]) ||
+	for (result = dns_rdataset_first(keyrdataset); result == ISC_R_SUCCESS;
+	     result = dns_rdataset_next(keyrdataset))
+	{
+		dns_rdata_t rdata = DNS_RDATA_INIT;
+		dns_rdataset_current(keyrdataset, &rdata);
+		result = dns_rdata_tostruct(&rdata, &key, NULL);
+		RUNTIME_CHECK(result == ISC_R_SUCCESS);
+		result = dns_dnssec_keyfromrdata(vctx->origin, &rdata,
+						 vctx->mctx, &dstkey);
+		if (result != ISC_R_SUCCESS) {
+			return (false);
+		}
+		if (sig.algorithm != key.algorithm ||
+		    sig.keyid != dst_key_id(dstkey) ||
 		    !dns_name_equal(&sig.signer, vctx->origin))
 		{
+			dst_key_free(&dstkey);
 			continue;
 		}
-		result = dns_dnssec_verify(name, rdataset, dstkeys[key], false,
-					   0, vctx->mctx, sigrdata, NULL);
+		result = dns_dnssec_verify(name, rdataset, dstkey, false, 0,
+					   vctx->mctx, sigrdata, NULL);
+		dst_key_free(&dstkey);
 		if (result == ISC_R_SUCCESS || result == DNS_R_FROMWILDCARD) {
 			return (true);
 		}
@@ -321,18 +325,18 @@ check_no_rrsig(const vctx_t *vctx, const dns_rdataset_t *rdataset,
 		if (sigrdataset.type == dns_rdatatype_rrsig &&
 		    sigrdataset.covers == rdataset->type)
 		{
-			dns_name_format(name, namebuf, sizeof(namebuf));
-			dns_rdatatype_format(rdataset->type, typebuf,
-					     sizeof(typebuf));
-			zoneverify_log_error(
-				vctx,
-				"Warning: Found unexpected signatures "
-				"for %s/%s",
-				namebuf, typebuf);
 			break;
 		}
 		dns_rdataset_disassociate(&sigrdataset);
 	}
+	if (result == ISC_R_SUCCESS) {
+		dns_name_format(name, namebuf, sizeof(namebuf));
+		dns_rdatatype_format(rdataset->type, typebuf, sizeof(typebuf));
+		zoneverify_log_error(vctx,
+				     "Warning: Found unexpected signatures "
+				     "for %s/%s",
+				     namebuf, typebuf);
+	}
 	if (dns_rdataset_isassociated(&sigrdataset)) {
 		dns_rdataset_disassociate(&sigrdataset);
 	}
@@ -344,6 +348,8 @@ check_no_rrsig(const vctx_t *vctx, const dns_rdataset_t *rdataset,
 static bool
 chain_compare(void *arg1, void *arg2) {
 	struct nsec3_chain_fixed *e1 = arg1, *e2 = arg2;
+	size_t len;
+
 	/*
 	 * Do each element in turn to get a stable sort.
 	 */
@@ -371,7 +377,8 @@ chain_compare(void *arg1, void *arg2) {
 	if (e1->next_length > e2->next_length) {
 		return (false);
 	}
-	if (memcmp(e1 + 1, e2 + 1, chain_length(e1)) < 0) {
+	len = e1->salt_length + 2 * e1->next_length;
+	if (memcmp(e1 + 1, e2 + 1, len) < 0) {
 		return (true);
 	}
 	return (false);
@@ -379,7 +386,9 @@ chain_compare(void *arg1, void *arg2) {
 
 static bool
 chain_equal(const struct nsec3_chain_fixed *e1,
-	    const struct nsec3_chain_fixed *e2, size_t data_length) {
+	    const struct nsec3_chain_fixed *e2) {
+	size_t len;
+
 	if (e1->hash != e2->hash) {
 		return (false);
 	}
@@ -392,16 +401,20 @@ chain_equal(const struct nsec3_chain_fixed *e1,
 	if (e1->next_length != e2->next_length) {
 		return (false);
 	}
-
-	return (memcmp(e1 + 1, e2 + 1, data_length) == 0);
+	len = e1->salt_length + 2 * e1->next_length;
+	if (memcmp(e1 + 1, e2 + 1, len) != 0) {
+		return (false);
+	}
+	return (true);
 }
 
-static void
+static isc_result_t
 record_nsec3(const vctx_t *vctx, const unsigned char *rawhash,
 	     const dns_rdata_nsec3_t *nsec3, isc_heap_t *chains) {
-	struct nsec3_chain_fixed *element = NULL;
-	unsigned char *cp = NULL;
+	struct nsec3_chain_fixed *element;
 	size_t len;
+	unsigned char *cp;
+	isc_result_t result;
 
 	len = sizeof(*element) + nsec3->next_length * 2 + nsec3->salt_length;
 
@@ -417,40 +430,12 @@ record_nsec3(const vctx_t *vctx, const unsigned char *rawhash,
 	memmove(cp, rawhash, nsec3->next_length);
 	cp += nsec3->next_length;
 	memmove(cp, nsec3->next, nsec3->next_length);
-	isc_heap_insert(chains, element);
-}
-
-/*
- * Check whether any NSEC3 within 'rdataset' matches the parameters in
- * 'nsec3param'.
- */
-static isc_result_t
-find_nsec3_match(const dns_rdata_nsec3param_t *nsec3param,
-		 dns_rdataset_t *rdataset, size_t rhsize,
-		 dns_rdata_nsec3_t *nsec3_match) {
-	isc_result_t result;
-
-	/*
-	 * Find matching NSEC3 record.
-	 */
-	for (result = dns_rdataset_first(rdataset); result == ISC_R_SUCCESS;
-	     result = dns_rdataset_next(rdataset))
-	{
-		dns_rdata_t rdata = DNS_RDATA_INIT;
-		dns_rdataset_current(rdataset, &rdata);
-		result = dns_rdata_tostruct(&rdata, nsec3_match, NULL);
-		RUNTIME_CHECK(result == ISC_R_SUCCESS);
-		if (nsec3_match->hash == nsec3param->hash &&
-		    nsec3_match->next_length == rhsize &&
-		    nsec3_match->iterations == nsec3param->iterations &&
-		    nsec3_match->salt_length == nsec3param->salt_length &&
-		    memcmp(nsec3_match->salt, nsec3param->salt,
-			   nsec3param->salt_length) == 0)
-		{
-			return (ISC_R_SUCCESS);
-		}
+	result = isc_heap_insert(chains, element);
+	if (result != ISC_R_SUCCESS) {
+		zoneverify_log_error(vctx, "isc_heap_insert failed: %s",
+				     isc_result_totext(result));
+		isc_mem_put(vctx->mctx, element, len);
 	}
-
 	return (result);
 }
 
@@ -466,7 +451,26 @@ match_nsec3(const vctx_t *vctx, const dns_name_t *name,
 	isc_result_t result;
 	unsigned int len;
 
-	result = find_nsec3_match(nsec3param, rdataset, rhsize, &nsec3);
+	/*
+	 * Find matching NSEC3 record.
+	 */
+	for (result = dns_rdataset_first(rdataset); result == ISC_R_SUCCESS;
+	     result = dns_rdataset_next(rdataset))
+	{
+		dns_rdata_t rdata = DNS_RDATA_INIT;
+		dns_rdataset_current(rdataset, &rdata);
+		result = dns_rdata_tostruct(&rdata, &nsec3, NULL);
+		RUNTIME_CHECK(result == ISC_R_SUCCESS);
+		if (nsec3.hash == nsec3param->hash &&
+		    nsec3.next_length == rhsize &&
+		    nsec3.iterations == nsec3param->iterations &&
+		    nsec3.salt_length == nsec3param->salt_length &&
+		    memcmp(nsec3.salt, nsec3param->salt,
+			   nsec3param->salt_length) == 0)
+		{
+			break;
+		}
+	}
 	if (result != ISC_R_SUCCESS) {
 		dns_name_format(name, namebuf, sizeof(namebuf));
 		zoneverify_log_error(vctx, "Missing NSEC3 record for %s",
@@ -492,7 +496,12 @@ match_nsec3(const vctx_t *vctx, const dns_name_t *name,
 	/*
 	 * Record chain.
 	 */
-	record_nsec3(vctx, rawhash, &nsec3, vctx->expected_chains);
+	result = record_nsec3(vctx, rawhash, &nsec3, vctx->expected_chains);
+	if (result != ISC_R_SUCCESS) {
+		zoneverify_log_error(vctx, "record_nsec3(): %s",
+				     isc_result_totext(result));
+		return (result);
+	}
 
 	/*
 	 * Make sure there is only one NSEC3 record with this set of
@@ -596,7 +605,6 @@ record_found(const vctx_t *vctx, const dns_name_t *name, dns_dbnode_t *node,
 		if (nsec3.next_length != isc_buffer_usedlength(&b)) {
 			continue;
 		}
-
 		/*
 		 * We only care about NSEC3 records that match a NSEC3PARAM
 		 * record.
@@ -608,7 +616,12 @@ record_found(const vctx_t *vctx, const dns_name_t *name, dns_dbnode_t *node,
 		/*
 		 * Record chain.
 		 */
-		record_nsec3(vctx, owner, &nsec3, vctx->found_chains);
+		result = record_nsec3(vctx, owner, &nsec3, vctx->found_chains);
+		if (result != ISC_R_SUCCESS) {
+			zoneverify_log_error(vctx, "record_nsec3(): %s",
+					     isc_result_totext(result));
+			goto cleanup;
+		}
 	}
 	result = ISC_R_SUCCESS;
 
@@ -618,11 +631,11 @@ cleanup:
 }
 
 static isc_result_t
-isoptout(const vctx_t *vctx, const dns_rdata_nsec3param_t *nsec3param,
-	 bool *optout) {
+isoptout(const vctx_t *vctx, const dns_rdata_t *nsec3rdata, bool *optout) {
 	dns_rdataset_t rdataset;
 	dns_rdata_t rdata = DNS_RDATA_INIT;
 	dns_rdata_nsec3_t nsec3;
+	dns_rdata_nsec3param_t nsec3param;
 	dns_fixedname_t fixed;
 	dns_name_t *hashname;
 	isc_result_t result;
@@ -630,11 +643,14 @@ isoptout(const vctx_t *vctx, const dns_rdata_nsec3param_t *nsec3param,
 	unsigned char rawhash[NSEC3_MAX_HASH_LENGTH];
 	size_t rhsize = sizeof(rawhash);
 
+	result = dns_rdata_tostruct(nsec3rdata, &nsec3param, NULL);
+	RUNTIME_CHECK(result == ISC_R_SUCCESS);
+
 	dns_fixedname_init(&fixed);
 	result = dns_nsec3_hashname(&fixed, rawhash, &rhsize, vctx->origin,
-				    vctx->origin, nsec3param->hash,
-				    nsec3param->iterations, nsec3param->salt,
-				    nsec3param->salt_length);
+				    vctx->origin, nsec3param.hash,
+				    nsec3param.iterations, nsec3param.salt,
+				    nsec3param.salt_length);
 	if (result != ISC_R_SUCCESS) {
 		zoneverify_log_error(vctx, "dns_nsec3_hashname(): %s",
 				     isc_result_totext(result));
@@ -707,14 +723,7 @@ verifynsec3(const vctx_t *vctx, const dns_name_t *name,
 		return (ISC_R_SUCCESS);
 	}
 
-	if (nsec3param.iterations > DNS_NSEC3_MAXITERATIONS) {
-		result = DNS_R_NSEC3ITERRANGE;
-		zoneverify_log_error(vctx, "verifynsec3: %s",
-				     isc_result_totext(result));
-		return (result);
-	}
-
-	result = isoptout(vctx, &nsec3param, &optout);
+	result = isoptout(vctx, rdata, &optout);
 	if (result != ISC_R_SUCCESS) {
 		return (result);
 	}
@@ -807,14 +816,15 @@ verifynsec3s(const vctx_t *vctx, const dns_name_t *name,
 
 static isc_result_t
 verifyset(vctx_t *vctx, dns_rdataset_t *rdataset, const dns_name_t *name,
-	  dns_dbnode_t *node, dst_key_t **dstkeys, size_t nkeys) {
-	unsigned char set_algorithms[256] = { 0 };
+	  dns_dbnode_t *node, dns_rdataset_t *keyrdataset) {
+	unsigned char set_algorithms[256];
 	char namebuf[DNS_NAME_FORMATSIZE];
 	char algbuf[DNS_SECALG_FORMATSIZE];
 	char typebuf[DNS_RDATATYPE_FORMATSIZE];
 	dns_rdataset_t sigrdataset;
 	dns_rdatasetiter_t *rdsiter = NULL;
 	isc_result_t result;
+	int i;
 
 	dns_rdataset_init(&sigrdataset);
 	result = dns_db_allrdatasets(vctx->db, node, vctx->ver, 0, &rdsiter);
@@ -839,7 +849,7 @@ verifyset(vctx_t *vctx, dns_rdataset_t *rdataset, const dns_name_t *name,
 		dns_rdatatype_format(rdataset->type, typebuf, sizeof(typebuf));
 		zoneverify_log_error(vctx, "No signatures for %s/%s", namebuf,
 				     typebuf);
-		for (size_t i = 0; i < ARRAY_SIZE(set_algorithms); i++) {
+		for (i = 0; i < 256; i++) {
 			if (vctx->act_algorithms[i] != 0) {
 				vctx->bad_algorithms[i] = 1;
 			}
@@ -848,6 +858,7 @@ verifyset(vctx_t *vctx, dns_rdataset_t *rdataset, const dns_name_t *name,
 		goto done;
 	}
 
+	memset(set_algorithms, 0, sizeof(set_algorithms));
 	for (result = dns_rdataset_first(&sigrdataset); result == ISC_R_SUCCESS;
 	     result = dns_rdataset_next(&sigrdataset))
 	{
@@ -872,7 +883,7 @@ verifyset(vctx_t *vctx, dns_rdataset_t *rdataset, const dns_name_t *name,
 		{
 			continue;
 		}
-		if (goodsig(vctx, &rdata, name, dstkeys, nkeys, rdataset)) {
+		if (goodsig(vctx, &rdata, name, keyrdataset, rdataset)) {
 			dns_rdataset_settrust(rdataset, dns_trust_secure);
 			dns_rdataset_settrust(&sigrdataset, dns_trust_secure);
 			set_algorithms[sig.algorithm] = 1;
@@ -881,10 +892,10 @@ verifyset(vctx_t *vctx, dns_rdataset_t *rdataset, const dns_name_t *name,
 	result = ISC_R_SUCCESS;
 
 	if (memcmp(set_algorithms, vctx->act_algorithms,
-		   sizeof(set_algorithms)) != 0) {
+		   sizeof(set_algorithms))) {
 		dns_name_format(name, namebuf, sizeof(namebuf));
 		dns_rdatatype_format(rdataset->type, typebuf, sizeof(typebuf));
-		for (size_t i = 0; i < ARRAY_SIZE(set_algorithms); i++) {
+		for (i = 0; i < 256; i++) {
 			if ((vctx->act_algorithms[i] != 0) &&
 			    (set_algorithms[i] == 0)) {
 				dns_secalg_format(i, algbuf, sizeof(algbuf));
@@ -908,10 +919,10 @@ done:
 
 static isc_result_t
 verifynode(vctx_t *vctx, const dns_name_t *name, dns_dbnode_t *node,
-	   bool delegation, dst_key_t **dstkeys, size_t nkeys,
+	   bool delegation, dns_rdataset_t *keyrdataset,
 	   dns_rdataset_t *nsecset, dns_rdataset_t *nsec3paramset,
 	   const dns_name_t *nextname, isc_result_t *vresult) {
-	unsigned char types[8192] = { 0 };
+	unsigned char types[8192];
 	unsigned int maxtype = 0;
 	dns_rdataset_t rdataset;
 	dns_rdatasetiter_t *rdsiter = NULL;
@@ -919,13 +930,13 @@ verifynode(vctx_t *vctx, const dns_name_t *name, dns_dbnode_t *node,
 
 	REQUIRE(vresult != NULL || (nsecset == NULL && nsec3paramset == NULL));
 
+	memset(types, 0, sizeof(types));
 	result = dns_db_allrdatasets(vctx->db, node, vctx->ver, 0, &rdsiter);
 	if (result != ISC_R_SUCCESS) {
 		zoneverify_log_error(vctx, "dns_db_allrdatasets(): %s",
 				     isc_result_totext(result));
 		return (result);
 	}
-
 	result = dns_rdatasetiter_first(rdsiter);
 	dns_rdataset_init(&rdataset);
 	while (result == ISC_R_SUCCESS) {
@@ -942,8 +953,8 @@ verifynode(vctx_t *vctx, const dns_name_t *name, dns_dbnode_t *node,
 		    (!delegation || rdataset.type == dns_rdatatype_ds ||
 		     rdataset.type == dns_rdatatype_nsec))
 		{
-			result = verifyset(vctx, &rdataset, name, node, dstkeys,
-					   nkeys);
+			result = verifyset(vctx, &rdataset, name, node,
+					   keyrdataset);
 			if (result != ISC_R_SUCCESS) {
 				dns_rdataset_disassociate(&rdataset);
 				dns_rdatasetiter_destroy(&rdsiter);
@@ -1049,6 +1060,19 @@ check_no_nsec(const vctx_t *vctx, const dns_name_t *name, dns_dbnode_t *node) {
 	return (nsec_exists ? ISC_R_FAILURE : ISC_R_SUCCESS);
 }
 
+static bool
+newchain(const struct nsec3_chain_fixed *first,
+	 const struct nsec3_chain_fixed *e) {
+	if (first->hash != e->hash || first->iterations != e->iterations ||
+	    first->salt_length != e->salt_length ||
+	    first->next_length != e->next_length ||
+	    memcmp(first + 1, e + 1, first->salt_length) != 0)
+	{
+		return (true);
+	}
+	return (false);
+}
+
 static void
 free_element(isc_mem_t *mctx, struct nsec3_chain_fixed *e) {
 	size_t len;
@@ -1105,7 +1129,7 @@ _checknext(const vctx_t *vctx, const struct nsec3_chain_fixed *first,
 	return (false);
 }
 
-static bool
+static inline bool
 checknext(isc_mem_t *mctx, const vctx_t *vctx,
 	  const struct nsec3_chain_fixed *first, struct nsec3_chain_fixed *prev,
 	  const struct nsec3_chain_fixed *cur) {
@@ -1118,7 +1142,7 @@ checknext(isc_mem_t *mctx, const vctx_t *vctx,
 	return (result);
 }
 
-static bool
+static inline bool
 checklast(isc_mem_t *mctx, const vctx_t *vctx, struct nsec3_chain_fixed *first,
 	  struct nsec3_chain_fixed *prev) {
 	bool result = _checknext(vctx, prev, first);
@@ -1147,7 +1171,7 @@ verify_nsec3_chains(const vctx_t *vctx, isc_mem_t *mctx) {
 			/*
 			 * Check that they match.
 			 */
-			if (chain_equal(e, f, chain_length(e))) {
+			if (chain_equal(e, f)) {
 				free_element(mctx, f);
 				f = NULL;
 			} else {
@@ -1170,9 +1194,7 @@ verify_nsec3_chains(const vctx_t *vctx, isc_mem_t *mctx) {
 						isc_heap_delete(
 							vctx->found_chains, 1);
 					}
-					if (f != NULL &&
-					    chain_equal(e, f, chain_length(e)))
-					{
+					if (f != NULL && chain_equal(e, f)) {
 						free_element(mctx, f);
 						f = NULL;
 						break;
@@ -1188,7 +1210,7 @@ verify_nsec3_chains(const vctx_t *vctx, isc_mem_t *mctx) {
 
 		if (first == NULL) {
 			prev = first = e;
-		} else if (!chain_equal(first, e, first->salt_length)) {
+		} else if (newchain(first, e)) {
 			if (!checklast(mctx, vctx, first, prev)) {
 				result = ISC_R_FAILURE;
 			}
@@ -1269,9 +1291,11 @@ verifyemptynodes(const vctx_t *vctx, const dns_name_t *name,
 	return (ISC_R_SUCCESS);
 }
 
-static void
+static isc_result_t
 vctx_init(vctx_t *vctx, isc_mem_t *mctx, dns_zone_t *zone, dns_db_t *db,
 	  dns_dbversion_t *ver, dns_name_t *origin, dns_keytable_t *secroots) {
+	isc_result_t result;
+
 	memset(vctx, 0, sizeof(*vctx));
 
 	vctx->mctx = mctx;
@@ -1293,11 +1317,21 @@ vctx_init(vctx_t *vctx, isc_mem_t *mctx, dns_zone_t *zone, dns_db_t *db,
 	dns_rdataset_init(&vctx->nsec3paramsigs);
 
 	vctx->expected_chains = NULL;
-	isc_heap_create(mctx, chain_compare, NULL, 1024,
-			&vctx->expected_chains);
+	result = isc_heap_create(mctx, chain_compare, NULL, 1024,
+				 &vctx->expected_chains);
+	if (result != ISC_R_SUCCESS) {
+		return (result);
+	}
 
 	vctx->found_chains = NULL;
-	isc_heap_create(mctx, chain_compare, NULL, 1024, &vctx->found_chains);
+	result = isc_heap_create(mctx, chain_compare, NULL, 1024,
+				 &vctx->found_chains);
+	if (result != ISC_R_SUCCESS) {
+		isc_heap_destroy(&vctx->expected_chains);
+		return (result);
+	}
+
+	return (result);
 }
 
 static void
@@ -1460,18 +1494,18 @@ check_dnskey_sigs(vctx_t *vctx, const dns_rdata_dnskey_t *dnskey,
 		    dns_dnssec_signs(keyrdata, vctx->origin, &vctx->soaset,
 				     &vctx->soasigs, false, vctx->mctx))
 		{
-			if (active_keys[dnskey->algorithm] != DNS_KEYALG_MAX) {
+			if (active_keys[dnskey->algorithm] != 255) {
 				active_keys[dnskey->algorithm]++;
 			}
 		} else {
-			if (standby_keys[dnskey->algorithm] != DNS_KEYALG_MAX) {
+			if (standby_keys[dnskey->algorithm] != 255) {
 				standby_keys[dnskey->algorithm]++;
 			}
 		}
 		return;
 	}
 
-	if (active_keys[dnskey->algorithm] != DNS_KEYALG_MAX) {
+	if (active_keys[dnskey->algorithm] != 255) {
 		active_keys[dnskey->algorithm]++;
 	}
 
@@ -1579,9 +1613,9 @@ check_dnskey(vctx_t *vctx) {
 		RUNTIME_CHECK(result == ISC_R_SUCCESS);
 		is_ksk = ((dnskey.flags & DNS_KEYFLAG_KSK) != 0);
 
-		if ((dnskey.flags & DNS_KEYOWNER_ZONE) != 0 &&
-		    (dnskey.flags & DNS_KEYFLAG_REVOKE) != 0)
-		{
+		if ((dnskey.flags & DNS_KEYOWNER_ZONE) == 0) {
+			/* Non zone key, skip. */
+		} else if ((dnskey.flags & DNS_KEYFLAG_REVOKE) != 0) {
 			if ((dnskey.flags & DNS_KEYFLAG_KSK) != 0 &&
 			    !dns_dnssec_selfsigns(&rdata, vctx->origin,
 						  &vctx->keyset, &vctx->keysigs,
@@ -1611,13 +1645,11 @@ check_dnskey(vctx_t *vctx) {
 				return (ISC_R_FAILURE);
 			}
 			if ((dnskey.flags & DNS_KEYFLAG_KSK) != 0 &&
-			    vctx->revoked_ksk[dnskey.algorithm] !=
-				    DNS_KEYALG_MAX)
+			    vctx->revoked_ksk[dnskey.algorithm] != 255)
 			{
 				vctx->revoked_ksk[dnskey.algorithm]++;
 			} else if ((dnskey.flags & DNS_KEYFLAG_KSK) == 0 &&
-				   vctx->revoked_zsk[dnskey.algorithm] !=
-					   DNS_KEYALG_MAX)
+				   vctx->revoked_zsk[dnskey.algorithm] != 255)
 			{
 				vctx->revoked_zsk[dnskey.algorithm]++;
 			}
@@ -1636,10 +1668,11 @@ determine_active_algorithms(vctx_t *vctx, bool ignore_kskflag,
 			    bool keyset_kskonly,
 			    void (*report)(const char *, ...)) {
 	char algbuf[DNS_SECALG_FORMATSIZE];
+	int i;
 
 	report("Verifying the zone using the following algorithms:");
 
-	for (size_t i = 0; i < ARRAY_SIZE(vctx->act_algorithms); i++) {
+	for (i = 0; i < 256; i++) {
 		if (ignore_kskflag) {
 			vctx->act_algorithms[i] = (vctx->ksk_algorithms[i] !=
 							   0 ||
@@ -1653,15 +1686,16 @@ determine_active_algorithms(vctx_t *vctx, bool ignore_kskflag,
 		}
 		if (vctx->act_algorithms[i] != 0) {
 			dns_secalg_format(i, algbuf, sizeof(algbuf));
-			report("- %s", algbuf);
+			report(" %s", algbuf);
 		}
 	}
+	report(".\n");
 
 	if (ignore_kskflag || keyset_kskonly) {
 		return;
 	}
 
-	for (size_t i = 0; i < ARRAY_SIZE(vctx->ksk_algorithms); i++) {
+	for (i = 0; i < 256; i++) {
 		/*
 		 * The counts should both be zero or both be non-zero.  Mark
 		 * the algorithm as bad if this is not met.
@@ -1691,8 +1725,6 @@ verify_nodes(vctx_t *vctx, isc_result_t *vresult) {
 	dns_name_t *name, *nextname, *prevname, *zonecut;
 	dns_dbnode_t *node = NULL, *nextnode;
 	dns_dbiterator_t *dbiter = NULL;
-	dst_key_t **dstkeys;
-	size_t count, nkeys = 0;
 	bool done = false;
 	isc_result_t tvresult = ISC_R_UNSET;
 	isc_result_t result;
@@ -1704,27 +1736,11 @@ verify_nodes(vctx_t *vctx, isc_result_t *vresult) {
 	dns_fixedname_init(&fzonecut);
 	zonecut = NULL;
 
-	count = dns_rdataset_count(&vctx->keyset);
-	dstkeys = isc_mem_get(vctx->mctx, sizeof(*dstkeys) * count);
-
-	for (result = dns_rdataset_first(&vctx->keyset);
-	     result == ISC_R_SUCCESS; result = dns_rdataset_next(&vctx->keyset))
-	{
-		dns_rdata_t rdata = DNS_RDATA_INIT;
-		dns_rdataset_current(&vctx->keyset, &rdata);
-		dstkeys[nkeys] = NULL;
-		result = dns_dnssec_keyfromrdata(vctx->origin, &rdata,
-						 vctx->mctx, &dstkeys[nkeys]);
-		if (result == ISC_R_SUCCESS) {
-			nkeys++;
-		}
-	}
-
 	result = dns_db_createiterator(vctx->db, DNS_DB_NONSEC3, &dbiter);
 	if (result != ISC_R_SUCCESS) {
 		zoneverify_log_error(vctx, "dns_db_createiterator(): %s",
 				     isc_result_totext(result));
-		goto done;
+		return (result);
 	}
 
 	result = dns_dbiterator_first(dbiter);
@@ -1824,9 +1840,9 @@ verify_nodes(vctx_t *vctx, isc_result_t *vresult) {
 			dns_db_detachnode(vctx->db, &node);
 			goto done;
 		}
-		result = verifynode(vctx, name, node, isdelegation, dstkeys,
-				    nkeys, &vctx->nsecset, &vctx->nsec3paramset,
-				    nextname, &tvresult);
+		result = verifynode(vctx, name, node, isdelegation,
+				    &vctx->keyset, &vctx->nsecset,
+				    &vctx->nsec3paramset, nextname, &tvresult);
 		if (result != ISC_R_SUCCESS) {
 			dns_db_detachnode(vctx->db, &node);
 			goto done;
@@ -1874,7 +1890,7 @@ verify_nodes(vctx_t *vctx, isc_result_t *vresult) {
 					     isc_result_totext(result));
 			goto done;
 		}
-		result = verifynode(vctx, name, node, false, dstkeys, nkeys,
+		result = verifynode(vctx, name, node, false, &vctx->keyset,
 				    NULL, NULL, NULL, NULL);
 		if (result != ISC_R_SUCCESS) {
 			zoneverify_log_error(vctx, "verifynode: %s",
@@ -1892,13 +1908,7 @@ verify_nodes(vctx_t *vctx, isc_result_t *vresult) {
 	result = ISC_R_SUCCESS;
 
 done:
-	while (nkeys-- > 0U) {
-		dst_key_free(&dstkeys[nkeys]);
-	}
-	isc_mem_put(vctx->mctx, dstkeys, sizeof(*dstkeys) * count);
-	if (dbiter != NULL) {
-		dns_dbiterator_destroy(&dbiter);
-	}
+	dns_dbiterator_destroy(&dbiter);
 
 	return (result);
 }
@@ -1907,8 +1917,9 @@ static isc_result_t
 check_bad_algorithms(const vctx_t *vctx, void (*report)(const char *, ...)) {
 	char algbuf[DNS_SECALG_FORMATSIZE];
 	bool first = true;
+	int i;
 
-	for (size_t i = 0; i < ARRAY_SIZE(vctx->bad_algorithms); i++) {
+	for (i = 0; i < 256; i++) {
 		if (vctx->bad_algorithms[i] == 0) {
 			continue;
 		}
@@ -1922,7 +1933,7 @@ check_bad_algorithms(const vctx_t *vctx, void (*report)(const char *, ...)) {
 	}
 
 	if (!first) {
-		report(".");
+		report(".\n");
 	}
 
 	return (first ? ISC_R_SUCCESS : ISC_R_FAILURE);
@@ -1932,9 +1943,10 @@ static void
 print_summary(const vctx_t *vctx, bool keyset_kskonly,
 	      void (*report)(const char *, ...)) {
 	char algbuf[DNS_SECALG_FORMATSIZE];
+	int i;
 
-	report("Zone fully signed:");
-	for (size_t i = 0; i < ARRAY_SIZE(vctx->ksk_algorithms); i++) {
+	report("Zone fully signed:\n");
+	for (i = 0; i < 256; i++) {
 		if ((vctx->ksk_algorithms[i] == 0) &&
 		    (vctx->standby_ksk[i] == 0) &&
 		    (vctx->revoked_ksk[i] == 0) &&
@@ -1945,11 +1957,11 @@ print_summary(const vctx_t *vctx, bool keyset_kskonly,
 		}
 		dns_secalg_format(i, algbuf, sizeof(algbuf));
 		report("Algorithm: %s: KSKs: "
-		       "%u active, %u stand-by, %u revoked",
+		       "%u active, %u stand-by, %u revoked\n",
 		       algbuf, vctx->ksk_algorithms[i], vctx->standby_ksk[i],
 		       vctx->revoked_ksk[i]);
 		report("%*sZSKs: "
-		       "%u active, %u %s, %u revoked",
+		       "%u active, %u %s, %u revoked\n",
 		       (int)strlen(algbuf) + 13, "", vctx->zsk_algorithms[i],
 		       vctx->standby_zsk[i],
 		       keyset_kskonly ? "present" : "stand-by",
@@ -1966,7 +1978,10 @@ dns_zoneverify_dnssec(dns_zone_t *zone, dns_db_t *db, dns_dbversion_t *ver,
 	isc_result_t result, vresult = ISC_R_UNSET;
 	vctx_t vctx;
 
-	vctx_init(&vctx, mctx, zone, db, ver, origin, secroots);
+	result = vctx_init(&vctx, mctx, zone, db, ver, origin, secroots);
+	if (result != ISC_R_SUCCESS) {
+		return (result);
+	}
 
 	result = check_apex_rrsets(&vctx);
 	if (result != ISC_R_SUCCESS) {
@@ -2009,13 +2024,13 @@ dns_zoneverify_dnssec(dns_zone_t *zone, dns_db_t *db, dns_dbversion_t *ver,
 
 	result = check_bad_algorithms(&vctx, report);
 	if (result != ISC_R_SUCCESS) {
-		report("DNSSEC completeness test failed.");
+		report("DNSSEC completeness test failed.\n");
 		goto done;
 	}
 
 	result = vresult;
 	if (result != ISC_R_SUCCESS) {
-		report("DNSSEC completeness test failed (%s).",
+		report("DNSSEC completeness test failed (%s).\n",
 		       dns_result_totext(result));
 		goto done;
 	}
