@@ -16,6 +16,7 @@
 #include <isc/atomic.h>
 #include <isc/buffer.h>
 #include <isc/condition.h>
+#include <isc/errno.h>
 #include <isc/log.h>
 #include <isc/magic.h>
 #include <isc/mem.h>
@@ -48,7 +49,7 @@ can_log_tcp_quota(void) {
 	return (false);
 }
 
-static int
+static isc_result_t
 tcp_connect_direct(isc_nmsocket_t *sock, isc__nm_uvreq_t *req);
 
 static void
@@ -76,23 +77,114 @@ accept_connection(isc_nmsocket_t *ssock, isc_quota_t *quota);
 static void
 quota_accept_cb(isc_quota_t *quota, void *sock0);
 
-static int
+static void
+failed_accept_cb(isc_nmsocket_t *sock, isc_result_t eresult);
+
+static void
+failed_send_cb(isc_nmsocket_t *sock, isc__nm_uvreq_t *req,
+	       isc_result_t eresult);
+
+static bool
+inactive(isc_nmsocket_t *sock) {
+	return (!isc__nmsocket_active(sock) ||
+		atomic_load(&sock->mgr->closing) ||
+		(sock->server != NULL && !isc__nmsocket_active(sock->server)));
+}
+
+static void
+failed_accept_cb(isc_nmsocket_t *sock, isc_result_t eresult) {
+	/*
+	 * Detach the quota early to make room for other connections;
+	 * otherwise it'd be detached later asynchronously, and clog
+	 * the quota unnecessarily.
+	 */
+	if (sock->quota != NULL) {
+		isc_quota_detach(&sock->quota);
+	}
+
+	if (!sock->accepting) {
+		return;
+	}
+	sock->accepting = false;
+
+	switch (eresult) {
+	case ISC_R_NOTCONNECTED:
+		/* IGNORE: The client disconnected before we could accept */
+		break;
+	default:
+		isc_log_write(isc_lctx, ISC_LOGCATEGORY_GENERAL,
+			      ISC_LOGMODULE_NETMGR, ISC_LOG_ERROR,
+			      "Accepting TCP connection failed: %s",
+			      isc_result_totext(eresult));
+	}
+
+	/*
+	 * Detach the socket properly to make sure uv_close() is called.
+	 */
+	isc__nmsocket_detach(&sock);
+}
+
+static void
+failed_connect_cb(isc_nmsocket_t *sock, isc__nm_uvreq_t *req,
+		  isc_result_t eresult) {
+	REQUIRE(VALID_NMSOCK(sock));
+	REQUIRE(VALID_UVREQ(req));
+	REQUIRE(sock->tid == isc_nm_tid());
+
+	if (sock->timer_running) {
+		uv_timer_stop(&sock->timer);
+		sock->timer_running = false;
+	}
+
+	if (!atomic_load(&sock->connecting)) {
+		isc__nm_uvreq_put(&req, sock);
+		return;
+	}
+	atomic_store(&sock->connecting, false);
+
+	isc__nmsocket_clearcb(sock);
+	if (req->cb.connect != NULL) {
+		isc__nm_connectcb(sock, req, eresult);
+	} else {
+		isc__nm_uvreq_put(&req, sock);
+	}
+}
+
+static void
+connecttimeout_cb(uv_timer_t *handle) {
+	isc__nm_uvreq_t *req = uv_handle_get_data((uv_handle_t *)handle);
+	isc_nmsocket_t *sock = req->sock;
+
+	REQUIRE(VALID_UVREQ(req));
+	REQUIRE(VALID_NMHANDLE(req->handle));
+	REQUIRE(sock->tid == isc_nm_tid());
+
+	failed_connect_cb(sock, req, ISC_R_TIMEDOUT);
+	isc__nmsocket_detach(&sock);
+}
+
+static isc_result_t
 tcp_connect_direct(isc_nmsocket_t *sock, isc__nm_uvreq_t *req) {
 	isc__networker_t *worker = NULL;
 	int r;
 
-	REQUIRE(isc__nm_in_netthread());
+	REQUIRE(VALID_NMSOCK(sock));
+	REQUIRE(VALID_UVREQ(req));
 
-	worker = &sock->mgr->workers[isc_nm_tid()];
+	REQUIRE(isc__nm_in_netthread());
+	REQUIRE(sock->tid == isc_nm_tid());
+
+	worker = &sock->mgr->workers[sock->tid];
+
+	atomic_store(&sock->connecting, true);
 
 	r = uv_tcp_init(&worker->loop, &sock->uv_handle.tcp);
 	if (r != 0) {
 		isc__nm_incstats(sock->mgr, sock->statsindex[STATID_OPENFAIL]);
-		/* Socket was never opened; no need for tcp_close_direct() */
+		atomic_store(&sock->closing, true);
 		atomic_store(&sock->closed, true);
-		sock->result = isc__nm_uverr2result(r);
-		atomic_store(&sock->connect_error, true);
-		return (r);
+		atomic_store(&sock->active, false);
+		return (isc__nm_uverr2result(r));
 	}
 
 	if (req->local.length != 0) {
@@ -100,24 +192,36 @@ tcp_connect_direct(isc_nmsocket_t *sock, isc__nm_uvreq_t *req) {
 		if (r != 0) {
 			isc__nm_incstats(sock->mgr,
 					 sock->statsindex[STATID_BINDFAIL]);
-			sock->result = isc__nm_uverr2result(r);
-			atomic_store(&sock->connect_error, true);
-			tcp_close_direct(sock);
-			return (r);
+			atomic_store(&sock->active, false);
+			isc__nm_tcp_close(sock);
+			return (isc__nm_uverr2result(r));
 		}
 	}
 
+	if (!sock->timer_initialized) {
+		uv_timer_init(&worker->loop, &sock->timer);
+		uv_handle_set_data((uv_handle_t *)&sock->timer, req);
+		sock->timer_initialized = true;
+	}
+
 	uv_handle_set_data(&sock->uv_handle.handle, sock);
+	uv_handle_set_data(&req->uv_req.handle, req);
 	r = uv_tcp_connect(&req->uv_req.connect, &sock->uv_handle.tcp,
 			   &req->peer.type.sa, tcp_connect_cb);
 	if (r != 0) {
 		isc__nm_incstats(sock->mgr,
 				 sock->statsindex[STATID_CONNECTFAIL]);
-		sock->result = isc__nm_uverr2result(r);
-		atomic_store(&sock->connect_error, true);
-		tcp_close_direct(sock);
+		atomic_store(&sock->active, false);
+		isc__nm_tcp_close(sock);
+		return (isc__nm_uverr2result(r));
 	}
-	return (r);
+	isc__nm_incstats(sock->mgr, sock->statsindex[STATID_CONNECT]);
+
+	uv_timer_start(&sock->timer, connecttimeout_cb, sock->connect_timeout,
+		       0);
+	sock->timer_running = true;
+
+	return (ISC_R_SUCCESS);
 }
 
 void
@@ -126,20 +230,29 @@ isc__nm_async_tcpconnect(isc__networker_t *worker, isc__netievent_t *ev0) {
 		(isc__netievent_tcpconnect_t *)ev0;
 	isc_nmsocket_t *sock = ievent->sock;
 	isc__nm_uvreq_t *req = ievent->req;
-	int r;
+	isc_result_t result = ISC_R_SUCCESS;
 
 	UNUSED(worker);
 
-	r = tcp_connect_direct(sock, req);
-	if (r != 0) {
-		/* We need to issue callbacks ourselves */
-		tcp_connect_cb(&req->uv_req.connect, r);
-		goto done;
+	REQUIRE(VALID_NMSOCK(sock));
+	REQUIRE(sock->type == isc_nm_tcpsocket);
+	REQUIRE(sock->iface != NULL);
+	REQUIRE(sock->parent == NULL);
+	REQUIRE(sock->tid == isc_nm_tid());
+
+	req->handle = isc__nmhandle_get(sock, &req->peer, &sock->iface->addr);
+	result = tcp_connect_direct(sock, req);
+	atomic_store(&sock->result, result);
+	if (result == ISC_R_SUCCESS) {
+		atomic_store(&sock->connected, true);
+		/* uvreq will be freed in tcp_connect_cb */
+		/* socket will be detached in tcp_connect_cb */
+	} else {
+		atomic_store(&sock->connect_error, true);
+		isc__nm_uvreq_put(&req, sock);
+		isc__nmsocket_detach(&ievent->sock);
 	}
 
-	atomic_store(&sock->connected, true);
-
-done:
 	LOCK(&sock->lock);
 	SIGNAL(&sock->cond);
 	UNLOCK(&sock->lock);
@@ -148,102 +261,114 @@ done:
 static void
 tcp_connect_cb(uv_connect_t *uvreq, int status) {
 	isc_result_t result;
-	isc__nm_uvreq_t *req = (isc__nm_uvreq_t *)uvreq->data;
-	isc_nmsocket_t *sock = NULL;
+	isc__nm_uvreq_t *req = NULL;
+	isc_nmsocket_t *sock = uv_handle_get_data((uv_handle_t *)uvreq->handle);
 	struct sockaddr_storage ss;
-	isc_nmhandle_t *handle = NULL;
+	int r;
 
-	sock = uv_handle_get_data((uv_handle_t *)uvreq->handle);
+	REQUIRE(VALID_NMSOCK(sock));
+	REQUIRE(sock->tid == isc_nm_tid());
 
+	/* We timed out */
+	if (!atomic_load(&sock->connecting)) {
+		return;
+	}
+
+	req = uv_handle_get_data((uv_handle_t *)uvreq);
+
 	REQUIRE(VALID_UVREQ(req));
+	REQUIRE(VALID_NMHANDLE(req->handle));
+
+	if (sock->timer_running) {
+		uv_timer_stop(&sock->timer);
+		sock->timer_running = false;
+	}
 
 	if (status != 0) {
-		req->cb.connect(NULL, isc__nm_uverr2result(status), req->cbarg);
-		isc__nm_uvreq_put(&req, sock);
+		failed_connect_cb(sock, req, isc__nm_uverr2result(status));
+		isc__nmsocket_detach(&sock);
 		return;
 	}
 
-	sock = uv_handle_get_data((uv_handle_t *)uvreq->handle);
 	isc__nm_incstats(sock->mgr, sock->statsindex[STATID_CONNECT]);
-	uv_tcp_getpeername(&sock->uv_handle.tcp, (struct sockaddr *)&ss,
-			   &(int){ sizeof(ss) });
+	r = uv_tcp_getpeername(&sock->uv_handle.tcp, (struct sockaddr *)&ss,
+			       &(int){ sizeof(ss) });
+	if (r != 0) {
+		failed_connect_cb(sock, req, isc__nm_uverr2result(r));
+		isc__nmsocket_detach(&sock);
+		return;
+	}
+
+	atomic_store(&sock->connecting, false);
+
 	result = isc_sockaddr_fromsockaddr(&sock->peer, (struct sockaddr *)&ss);
 	RUNTIME_CHECK(result == ISC_R_SUCCESS);
 
-	handle = isc__nmhandle_get(sock, NULL, NULL);
-	req->cb.connect(handle, ISC_R_SUCCESS, req->cbarg);
-
-	isc__nm_uvreq_put(&req, sock);
-
-	atomic_init(&sock->client, true);
+	isc__nm_connectcb(sock, req, ISC_R_SUCCESS);
 
 	/*
 	 * The sock is now attached to the handle.
 	 */
 	isc__nmsocket_detach(&sock);
-
-	/*
-	 * The connect callback should have attached to the handle.
-	 * If it didn't, the socket will be closed now.
-	 */
-	isc_nmhandle_detach(&handle);
 }
 
 isc_result_t
 isc_nm_tcpconnect(isc_nm_t *mgr, isc_nmiface_t *local, isc_nmiface_t *peer,
-		  isc_nm_cb_t cb, void *cbarg, size_t extrahandlesize) {
-	isc_nmsocket_t *nsock = NULL, *tmp = NULL;
+		  isc_nm_cb_t cb, void *cbarg, unsigned int timeout,
+		  size_t extrahandlesize) {
+	isc_result_t result = ISC_R_SUCCESS;
+	isc_nmsocket_t *sock = NULL, *tmp = NULL;
 	isc__netievent_tcpconnect_t *ievent = NULL;
 	isc__nm_uvreq_t *req = NULL;
-	isc_result_t result = ISC_R_SUCCESS;
 
 	REQUIRE(VALID_NM(mgr));
 	REQUIRE(local != NULL);
 	REQUIRE(peer != NULL);
 
-	nsock = isc_mem_get(mgr->mctx, sizeof(*nsock));
-	isc__nmsocket_init(nsock, mgr, isc_nm_tcpsocket, local);
-	nsock->extrahandlesize = extrahandlesize;
-	nsock->result = ISC_R_SUCCESS;
+	sock = isc_mem_get(mgr->mctx, sizeof(*sock));
+	isc__nmsocket_init(sock, mgr, isc_nm_tcpsocket, local);
 
-	req = isc__nm_uvreq_get(mgr, nsock);
+	sock->extrahandlesize = extrahandlesize;
+	sock->connect_timeout = timeout;
+
+	atomic_init(&sock->result, ISC_R_SUCCESS);
+	atomic_init(&sock->client, true);
+
+	req = isc__nm_uvreq_get(mgr, sock);
 	req->cb.connect = cb;
 	req->cbarg = cbarg;
 	req->peer = peer->addr;
 	req->local = local->addr;
 
 	ievent = isc__nm_get_ievent(mgr, netievent_tcpconnect);
-	ievent->sock = nsock;
+	ievent->sock = sock;
 	ievent->req = req;
 
 	/*
 	 * Async callbacks can dereference the socket in the meantime,
 	 * we need to hold an additional reference to it.
 	 */
-	isc__nmsocket_attach(nsock, &tmp);
+	isc__nmsocket_attach(sock, &tmp);
 
 	if (isc__nm_in_netthread()) {
-		nsock->tid = isc_nm_tid();
-		isc__nm_async_tcpconnect(&mgr->workers[nsock->tid],
+		sock->tid = isc_nm_tid();
+		isc__nm_async_tcpconnect(&mgr->workers[sock->tid],
 					 (isc__netievent_t *)ievent);
 		isc__nm_put_ievent(mgr, ievent);
 	} else {
-		nsock->tid = isc_random_uniform(mgr->nworkers);
-		isc__nm_enqueue_ievent(&mgr->workers[nsock->tid],
+		sock->tid = isc_random_uniform(mgr->nworkers);
+		isc__nm_enqueue_ievent(&mgr->workers[sock->tid],
 				       (isc__netievent_t *)ievent);
 
-		LOCK(&nsock->lock);
-		while (!atomic_load(&nsock->connected) &&
-		       !atomic_load(&nsock->connect_error)) {
-			WAIT(&nsock->cond, &nsock->lock);
+		LOCK(&sock->lock);
+		while (!atomic_load(&sock->connected) &&
+		       !atomic_load(&sock->connect_error)) {
+			WAIT(&sock->cond, &sock->lock);
 		}
-		UNLOCK(&nsock->lock);
+		UNLOCK(&sock->lock);
 	}
 
-	if (nsock->result != ISC_R_SUCCESS) {
-		result = nsock->result;
-		isc__nmsocket_detach(&nsock);
-	}
+	result = atomic_load(&sock->result);
 
 	isc__nmsocket_detach(&tmp);
 
@@ -262,11 +387,12 @@ isc_nm_listentcp(isc_nm_t *mgr, isc_nmiface_t *iface,
 
 	nsock = isc_mem_get(mgr->mctx, sizeof(*nsock));
 	isc__nmsocket_init(nsock, mgr, isc_nm_tcplistener, iface);
+
 	nsock->accept_cb = accept_cb;
 	nsock->accept_cbarg = accept_cbarg;
 	nsock->extrahandlesize = extrahandlesize;
 	nsock->backlog = backlog;
-	nsock->result = ISC_R_SUCCESS;
+	atomic_init(&nsock->result, ISC_R_SUCCESS);
 	if (quota != NULL) {
 		/*
 		 * We don't attach to quota, just assign - to avoid
@@ -296,11 +422,11 @@ isc_nm_listentcp(isc_nm_t *mgr, isc_nmiface_t *iface,
 		UNLOCK(&nsock->lock);
 	}
 
-	if (nsock->result == ISC_R_SUCCESS) {
+	if (atomic_load(&nsock->result) == ISC_R_SUCCESS) {
 		*sockp = nsock;
 		return (ISC_R_SUCCESS);
 	} else {
-		isc_result_t result = nsock->result;
+		isc_result_t result = atomic_load(&nsock->result);
 		isc__nmsocket_detach(&nsock);
 		return (result);
 	}
@@ -329,7 +455,7 @@ isc__nm_async_tcplisten(isc__networker_t *worker, isc__netievent_t *ev0) {
 		isc__nm_incstats(sock->mgr, sock->statsindex[STATID_OPENFAIL]);
 		/* The socket was never opened, so no need for uv_close() */
 		atomic_store(&sock->closed, true);
-		sock->result = isc__nm_uverr2result(r);
+		atomic_store(&sock->result, isc__nm_uverr2result(r));
 		atomic_store(&sock->listen_error, true);
 		goto done;
 	}
@@ -341,10 +467,38 @@ isc__nm_async_tcplisten(isc__networker_t *worker, isc__netievent_t *ev0) {
 		flags = UV_TCP_IPV6ONLY;
 	}
 
-	r = uv_tcp_bind(&sock->uv_handle.tcp, &sock->iface->addr.type.sa,
-			flags);
+	uv_tcp_bind(&sock->uv_handle.tcp, &sock->iface->addr.type.sa, flags);
+	r = uv_fileno(&sock->uv_handle.handle, (uv_os_fd_t *)&fd);
+	if (r != 0) {
+		isc__nm_incstats(sock->mgr, sock->statsindex[STATID_BINDFAIL]);
+		uv_close(&sock->uv_handle.handle, tcp_close_cb);
+		atomic_store(&sock->result, isc__nm_uverr2result(r));
+		atomic_store(&sock->listen_error, true);
+		goto done;
+	}
+
+	/*
+	 * uv_tcp_bind() uses a delayed error, initially returning
+	 * success even if bind() fails. By calling uv_tcp_getsockname()
+	 * here we can find out whether the bind() call was successful.
+	 */
+	r = uv_tcp_getsockname(&sock->uv_handle.tcp, (struct sockaddr *)&sname,
+			       &snamelen);
+
+	if (r == UV_EADDRINUSE && isc__nm_socket_reuse(fd) == ISC_R_SUCCESS &&
+	    isc__nm_socket_reuse_lb(fd) == ISC_R_SUCCESS)
+	{
+		/*
+		 * Retry bind() with REUSEADDR/REUSEPORT if the address
+		 * was in use.
+		 */
+		uv_tcp_bind(&sock->uv_handle.tcp, &sock->iface->addr.type.sa,
+			    flags);
+		r = uv_tcp_getsockname(&sock->uv_handle.tcp,
+				       (struct sockaddr *)&sname, &snamelen);
+	}
+
 	if (r == UV_EADDRNOTAVAIL &&
-	    uv_fileno(&sock->uv_handle.handle, (uv_os_fd_t *)&fd) == 0 &&
 	    isc__nm_socket_freebind(fd, sa_family) == ISC_R_SUCCESS)
 	{
 		/*
@@ -353,14 +507,16 @@ isc__nm_async_tcplisten(isc__networker_t *worker, isc__netievent_t *ev0) {
 		 * addresses which are reported by the route socket, although
 		 * named is not yet able to properly bind to them.
 		 */
-		r = uv_tcp_bind(&sock->uv_handle.tcp,
-				&sock->iface->addr.type.sa, flags);
+		uv_tcp_bind(&sock->uv_handle.tcp, &sock->iface->addr.type.sa,
+			    flags);
+		r = uv_tcp_getsockname(&sock->uv_handle.tcp,
+				       (struct sockaddr *)&sname, &snamelen);
 	}
 
 	if (r != 0) {
 		isc__nm_incstats(sock->mgr, sock->statsindex[STATID_BINDFAIL]);
 		uv_close(&sock->uv_handle.handle, tcp_close_cb);
-		sock->result = isc__nm_uverr2result(r);
+		atomic_store(&sock->result, isc__nm_uverr2result(r));
 		atomic_store(&sock->listen_error, true);
 		goto done;
 	}
@@ -375,7 +531,7 @@ isc__nm_async_tcplisten(isc__networker_t *worker, isc__netievent_t *ev0) {
 			       &snamelen);
 	if (r != 0) {
 		uv_close(&sock->uv_handle.handle, tcp_close_cb);
-		sock->result = isc__nm_uverr2result(r);
+		atomic_store(&sock->result, isc__nm_uverr2result(r));
 		atomic_store(&sock->listen_error, true);
 		goto done;
 	}
@@ -392,7 +548,7 @@ isc__nm_async_tcplisten(isc__networker_t *worker, isc__netievent_t *ev0) {
 			      "uv_listen failed: %s",
 			      isc_result_totext(isc__nm_uverr2result(r)));
 		uv_close(&sock->uv_handle.handle, tcp_close_cb);
-		sock->result = isc__nm_uverr2result(r);
+		atomic_store(&sock->result, isc__nm_uverr2result(r));
 		atomic_store(&sock->listen_error, true);
 		goto done;
 	}
@@ -432,17 +588,32 @@ isc__nm_async_tcpchildaccept(isc__networker_t *worker, isc__netievent_t *ev0) {
 	isc__netievent_tcpchildaccept_t *ievent =
 		(isc__netievent_tcpchildaccept_t *)ev0;
 	isc_nmsocket_t *sock = ievent->sock;
-	isc_nmhandle_t *handle;
 	isc_result_t result;
+	isc__nm_uvreq_t *req = NULL;
 	struct sockaddr_storage ss;
 	isc_sockaddr_t local;
 	int r;
-	isc_nm_accept_cb_t accept_cb;
-	void *accept_cbarg;
 
 	REQUIRE(isc__nm_in_netthread());
 	REQUIRE(sock->tid == isc_nm_tid());
 
+	if (!sock->accepting) {
+		return;
+	}
+
+	/* Socket was closed midflight by isc__nm_tcp_shutdown() */
+	if (!isc__nmsocket_active(sock)) {
+		failed_accept_cb(sock, ISC_R_CANCELED);
+		return;
+	}
+
+	INSIST(sock->server != NULL);
+
+	if (!isc__nmsocket_active(sock->server)) {
+		failed_accept_cb(sock, ISC_R_CANCELED);
+		return;
+	}
+
 	sock->quota = ievent->quota;
 	ievent->quota = NULL;
 
@@ -482,45 +653,28 @@ isc__nm_async_tcpchildaccept(isc__networker_t *worker, isc__netievent_t *ev0) {
 	if (result != ISC_R_SUCCESS) {
 		goto error;
 	}
-
-	handle = isc__nmhandle_get(sock, NULL, &local);
+	sock->accepting = false;
 
 	INSIST(sock->accept_cb != NULL);
-	accept_cb = sock->accept_cb;
-	accept_cbarg = sock->accept_cbarg;
 
 	sock->read_timeout = sock->mgr->init;
-	accept_cb(handle, ISC_R_SUCCESS, accept_cbarg);
+
+	req = isc__nm_uvreq_get(sock->mgr, sock);
+	req->handle = isc__nmhandle_get(sock, NULL, &local);
+	req->cb.accept = sock->accept_cb;
+	req->cbarg = sock->accept_cbarg;
+
+	isc__nm_acceptcb(sock, req, ISC_R_SUCCESS);
 
 	/*
 	 * sock is now attached to the handle.
 	 */
 	isc__nmsocket_detach(&sock);
 
-	/*
-	 * The accept callback should have attached to the handle.
-	 * If it didn't, the socket will be closed now.
-	 */
-	isc_nmhandle_detach(&handle);
 	return;
 
 error:
-	/*
-	 * Detach the quota early to make room for other connections;
-	 * otherwise it'd be detached later asynchronously, and clog
-	 * the quota unnecessarily.
-	 */
-	if (sock->quota != NULL) {
-		isc_quota_detach(&sock->quota);
-	}
-	isc_log_write(isc_lctx, ISC_LOGCATEGORY_GENERAL, ISC_LOGMODULE_NETMGR,
-		      ISC_LOG_ERROR, "Accepting TCP connection failed: %s",
-		      isc_result_totext(result));
-
-	/*
-	 * Detach the socket properly to make sure uv_close() is called.
-	 */
-	isc__nmsocket_detach(&sock);
+	failed_accept_cb(sock, result);
 }
 
 void
@@ -579,11 +733,50 @@ tcp_listenclose_cb(uv_handle_t *handle) {
 	isc__nmsocket_detach(&sock);
 }
 
+static void
+failed_read_cb(isc_nmsocket_t *sock, isc_result_t result) {
+	REQUIRE(VALID_NMSOCK(sock));
+	REQUIRE(sock->statichandle != NULL);
+
+	if (sock->timer_initialized) {
+		uv_timer_stop(&sock->timer);
+		sock->timer_running = false;
+	}
+
+	if (sock->quota) {
+		isc_quota_detach(&sock->quota);
+	}
+
+	uv_read_stop(&sock->uv_handle.stream);
+
+	if (sock->recv_cb != NULL) {
+		isc__nm_uvreq_t *req = isc__nm_uvreq_get(sock->mgr, sock);
+		isc_nmhandle_attach(sock->statichandle, &req->handle);
+		req->cb.recv = sock->recv_cb;
+		req->cbarg = sock->recv_cbarg;
+
+		isc__nmsocket_clearcb(sock);
+
+		isc__nm_readcb(sock, req, result);
+	}
+}
+
+static void
+failed_send_cb(isc_nmsocket_t *sock, isc__nm_uvreq_t *req,
+	       isc_result_t eresult) {
+	REQUIRE(VALID_NMSOCK(sock));
+	REQUIRE(VALID_UVREQ(req));
+
+	if (req->cb.send != NULL) {
+		isc__nm_sendcb(sock, req, eresult);
+	} else {
+		isc__nm_uvreq_put(&req, sock);
+	}
+}
+
 static void
 readtimeout_cb(uv_timer_t *handle) {
 	isc_nmsocket_t *sock = uv_handle_get_data((uv_handle_t *)handle);
-	isc_nm_recv_cb_t cb;
-	void *cbarg;
 
 	REQUIRE(VALID_NMSOCK(sock));
 	REQUIRE(sock->tid == isc_nm_tid());
@@ -594,40 +787,39 @@ readtimeout_cb(uv_timer_t *handle) {
 	 */
 	if (atomic_load(&sock->processing)) {
 		uv_timer_start(handle, readtimeout_cb, sock->read_timeout, 0);
+		sock->timer_running = true;
 		return;
 	}
 
 	/*
 	 * Timeout; stop reading and process whatever we have.
 	 */
-	uv_read_stop(&sock->uv_handle.stream);
-	if (sock->quota) {
-		isc_quota_detach(&sock->quota);
-	}
-
-	cb = sock->recv_cb;
-	cbarg = sock->recv_cbarg;
-	isc__nmsocket_clearcb(sock);
-
-	if (cb != NULL) {
-		cb(sock->statichandle, ISC_R_TIMEDOUT, NULL, cbarg);
-	}
+	failed_read_cb(sock, ISC_R_TIMEDOUT);
 }
 
-isc_result_t
+void
 isc__nm_tcp_read(isc_nmhandle_t *handle, isc_nm_recv_cb_t cb, void *cbarg) {
-	isc_nmsocket_t *sock = NULL;
+	isc_nmsocket_t *sock = handle->sock;
 	isc__netievent_startread_t *ievent = NULL;
 
 	REQUIRE(VALID_NMHANDLE(handle));
 	REQUIRE(VALID_NMSOCK(handle->sock));
 
-	sock = handle->sock;
-
-	REQUIRE(sock->tid == isc_nm_tid());
 	sock->recv_cb = cb;
 	sock->recv_cbarg = cbarg;
 
+	if (inactive(sock)) {
+		isc__nm_incstats(sock->mgr, sock->statsindex[STATID_RECVFAIL]);
+		failed_read_cb(sock, ISC_R_CANCELED);
+		return;
+	}
+
+	REQUIRE(sock->tid == isc_nm_tid());
+
+	sock->read_timeout = (atomic_load(&sock->keepalive)
+				      ? sock->mgr->keepalive
+				      : sock->mgr->idle);
+
 	ievent = isc__nm_get_ievent(sock->mgr, netievent_tcpstartread);
 	ievent->sock = sock;
 
@@ -640,7 +832,7 @@ isc__nm_tcp_read(isc_nmhandle_t *handle, isc_nm_recv_cb_t cb, void *cbarg) {
 				       (isc__netievent_t *)ievent);
 	}
 
-	return (ISC_R_SUCCESS);
+	return;
 }
 
 /*%<
@@ -674,33 +866,42 @@ isc__nm_async_tcp_startread(isc__networker_t *worker, isc__netievent_t *ev0) {
 	int r;
 
 	REQUIRE(worker->id == isc_nm_tid());
-	if (sock->read_timeout != 0) {
-		if (!sock->timer_initialized) {
-			uv_timer_init(&worker->loop, &sock->timer);
-			uv_handle_set_data((uv_handle_t *)&sock->timer, sock);
-			sock->timer_initialized = true;
-		}
-		uv_timer_start(&sock->timer, readtimeout_cb, sock->read_timeout,
-			       0);
+
+	if (inactive(sock)) {
+		failed_read_cb(sock, ISC_R_CANCELED);
+		return;
 	}
 
 	r = uv_read_start(&sock->uv_handle.stream, tcp_alloc_cb, read_cb);
 	if (r != 0) {
 		isc__nm_incstats(sock->mgr, sock->statsindex[STATID_RECVFAIL]);
+		failed_read_cb(sock, ISC_R_CANCELED);
+		return;
+	}
+
+	if (sock->read_timeout != 0) {
+		if (!sock->timer_initialized) {
+			uv_timer_init(&worker->loop, &sock->timer);
+			sock->timer_initialized = true;
+		}
+		uv_handle_set_data((uv_handle_t *)&sock->timer, sock);
+		uv_timer_start(&sock->timer, readtimeout_cb, sock->read_timeout,
+			       0);
+		sock->timer_running = true;
 	}
 }
 
-isc_result_t
+void
 isc__nm_tcp_pauseread(isc_nmsocket_t *sock) {
 	isc__netievent_pauseread_t *ievent = NULL;
 
 	REQUIRE(VALID_NMSOCK(sock));
 
-	if (atomic_load(&sock->readpaused)) {
-		return (ISC_R_SUCCESS);
+	if (!atomic_compare_exchange_strong(&sock->readpaused, &(bool){ false },
+					    true)) {
+		return;
 	}
 
-	atomic_store(&sock->readpaused, true);
 	ievent = isc__nm_get_ievent(sock->mgr, netievent_tcppauseread);
 	ievent->sock = sock;
 
@@ -713,7 +914,7 @@ isc__nm_tcp_pauseread(isc_nmsocket_t *sock) {
 				       (isc__netievent_t *)ievent);
 	}
 
-	return (ISC_R_SUCCESS);
+	return;
 }
 
 void
@@ -724,13 +925,14 @@ isc__nm_async_tcp_pauseread(isc__networker_t *worker, isc__netievent_t *ev0) {
 	REQUIRE(VALID_NMSOCK(sock));
 	REQUIRE(worker->id == isc_nm_tid());
 
-	if (sock->timer_initialized) {
+	if (sock->timer_running) {
 		uv_timer_stop(&sock->timer);
+		sock->timer_running = false;
 	}
 	uv_read_stop(&sock->uv_handle.stream);
 }
 
-isc_result_t
+void
 isc__nm_tcp_resumeread(isc_nmsocket_t *sock) {
 	isc__netievent_startread_t *ievent = NULL;
 
@@ -738,14 +940,18 @@ isc__nm_tcp_resumeread(isc_nmsocket_t *sock) {
 	REQUIRE(sock->tid == isc_nm_tid());
 
 	if (sock->recv_cb == NULL) {
-		return (ISC_R_CANCELED);
+		return;
 	}
 
-	if (!atomic_load(&sock->readpaused)) {
-		return (ISC_R_SUCCESS);
+	if (!isc__nmsocket_active(sock)) {
+		failed_read_cb(sock, ISC_R_CANCELED);
+		return;
 	}
 
-	atomic_store(&sock->readpaused, false);
+	if (!atomic_compare_exchange_strong(&sock->readpaused, &(bool){ true },
+					    false)) {
+		return;
+	}
 
 	ievent = isc__nm_get_ievent(sock->mgr, netievent_tcpstartread);
 	ievent->sock = sock;
@@ -758,62 +964,54 @@ isc__nm_tcp_resumeread(isc_nmsocket_t *sock) {
 		isc__nm_enqueue_ievent(&sock->mgr->workers[sock->tid],
 				       (isc__netievent_t *)ievent);
 	}
-
-	return (ISC_R_SUCCESS);
 }
 
 static void
 read_cb(uv_stream_t *stream, ssize_t nread, const uv_buf_t *buf) {
 	isc_nmsocket_t *sock = uv_handle_get_data((uv_handle_t *)stream);
-	isc_nm_recv_cb_t cb;
-	void *cbarg;
 
 	REQUIRE(VALID_NMSOCK(sock));
 	REQUIRE(sock->tid == isc_nm_tid());
 	REQUIRE(buf != NULL);
 
-	cb = sock->recv_cb;
-	cbarg = sock->recv_cbarg;
-
 	if (nread >= 0) {
-		isc_region_t region = { .base = (unsigned char *)buf->base,
-					.length = nread };
+		if (sock->recv_cb != NULL) {
+			isc__nm_uvreq_t *req = isc__nm_uvreq_get(sock->mgr,
+								 sock);
+			req->cb.recv = sock->recv_cb;
+			req->cbarg = sock->recv_cbarg;
+			isc_nmhandle_attach(sock->statichandle, &req->handle);
 
-		if (cb != NULL) {
-			cb(sock->statichandle, ISC_R_SUCCESS, &region, cbarg);
+			/*
+			 * The callback will be called synchronously because the
+			 * result is ISC_R_SUCCESS, so we don't need to retain
+			 * the buffer
+			 */
+			req->uvbuf.base = buf->base;
+			req->uvbuf.len = nread;
+
+			isc__nm_readcb(sock, req, ISC_R_SUCCESS);
 		}
 
-		sock->read_timeout = (atomic_load(&sock->keepalive)
-					      ? sock->mgr->keepalive
-					      : sock->mgr->idle);
-
 		if (sock->timer_initialized && sock->read_timeout != 0) {
 			/* The timer will be updated */
 			uv_timer_start(&sock->timer, readtimeout_cb,
 				       sock->read_timeout, 0);
+			sock->timer_running = true;
+		}
+	} else {
+		/*
+		 * This might happen if the inner socket is closing.  It means
+		 * that it's detached, so the socket will be closed.
+		 */
+		if (nread != UV_EOF) {
+			isc__nm_incstats(sock->mgr,
+					 sock->statsindex[STATID_RECVFAIL]);
 		}
 
-		isc__nm_free_uvbuf(sock, buf);
-		return;
+		failed_read_cb(sock, ISC_R_EOF);
 	}
-
 	isc__nm_free_uvbuf(sock, buf);
-
-	/*
-	 * This might happen if the inner socket is closing.  It means that
-	 * it's detached, so the socket will be closed.
-	 */
-	if (cb != NULL) {
-		isc__nm_incstats(sock->mgr, sock->statsindex[STATID_RECVFAIL]);
-		isc__nmsocket_clearcb(sock);
-		cb(sock->statichandle, ISC_R_EOF, NULL, cbarg);
-	}
-
-	/*
-	 * We don't need to clean up now; the socket will be closed and
-	 * resources and quota reclaimed when handle is freed in
-	 * isc__nm_tcp_close().
-	 */
 }
 
 static void
@@ -875,14 +1073,13 @@ accept_connection(isc_nmsocket_t *ssock, isc_quota_t *quota) {
 	isc__netievent_tcpchildaccept_t *event = NULL;
 	isc__networker_t *worker = NULL;
 	uv_tcp_t *uvstream = NULL;
+	isc_nmsocket_t *csock = NULL;
 	isc_mem_t *mctx = NULL;
 	int r, w;
 
 	REQUIRE(VALID_NMSOCK(ssock));
 
-	if (!atomic_load_relaxed(&ssock->active) ||
-	    atomic_load_relaxed(&ssock->mgr->closing))
-	{
+	if (!isc__nmsocket_active(ssock) || atomic_load(&ssock->mgr->closing)) {
 		/* We're closing, bail */
 		if (quota != NULL) {
 			isc_quota_detach(&quota);
@@ -931,7 +1128,9 @@ accept_connection(isc_nmsocket_t *ssock, isc_quota_t *quota) {
 	if (r != 0) {
 		result = isc__nm_uverr2result(r);
 		uv_close((uv_handle_t *)uvstream, free_uvtcpt);
-		isc_quota_detach(&quota);
+		if (quota != NULL) {
+			isc_quota_detach(&quota);
+		}
 		return (result);
 	}
 
@@ -940,21 +1139,29 @@ accept_connection(isc_nmsocket_t *ssock, isc_quota_t *quota) {
 	event = isc__nm_get_ievent(ssock->mgr, netievent_tcpchildaccept);
 
 	/* Duplicate the server socket */
-	isc_nmsocket_t *csock = isc_mem_get(ssock->mgr->mctx,
-					    sizeof(isc_nmsocket_t));
+	r = isc_uv_export((uv_stream_t *)uvstream, &event->streaminfo);
+	if (r != 0) {
+		result = isc_errno_toresult(errno);
+		uv_close((uv_handle_t *)uvstream, free_uvtcpt);
+		if (quota != NULL) {
+			isc_quota_detach(&quota);
+		}
+		isc__nm_put_ievent(ssock->mgr, event);
+		return (result);
+	}
+
+	csock = isc_mem_get(ssock->mgr->mctx, sizeof(isc_nmsocket_t));
 	isc__nmsocket_init(csock, ssock->mgr, isc_nm_tcpsocket, ssock->iface);
 	csock->tid = w;
 	csock->extrahandlesize = ssock->extrahandlesize;
 	isc__nmsocket_attach(ssock, &csock->server);
 	csock->accept_cb = ssock->accept_cb;
 	csock->accept_cbarg = ssock->accept_cbarg;
+	csock->accepting = true;
 
 	event->sock = csock;
 	event->quota = quota;
 
-	r = isc_uv_export((uv_stream_t *)uvstream, &event->streaminfo);
-	RUNTIME_CHECK(r == 0);
-
 	uv_close((uv_handle_t *)uvstream, free_uvtcpt);
 
 	if (w == isc_nm_tid()) {
@@ -969,7 +1176,7 @@ accept_connection(isc_nmsocket_t *ssock, isc_quota_t *quota) {
 	return (ISC_R_SUCCESS);
 }
 
-isc_result_t
+void
 isc__nm_tcp_send(isc_nmhandle_t *handle, isc_region_t *region, isc_nm_cb_t cb,
 		 void *cbarg) {
 	isc_nmsocket_t *sock = handle->sock;
@@ -981,16 +1188,29 @@ isc__nm_tcp_send(isc_nmhandle_t *handle, isc_region_t *region, isc_nm_cb_t cb,
 	uvreq = isc__nm_uvreq_get(sock->mgr, sock);
 	uvreq->uvbuf.base = (char *)region->base;
 	uvreq->uvbuf.len = region->length;
+
 	isc_nmhandle_attach(handle, &uvreq->handle);
+
 	uvreq->cb.send = cb;
 	uvreq->cbarg = cbarg;
 
+	if (inactive(sock)) {
+		isc__nm_incstats(sock->mgr, sock->statsindex[STATID_SENDFAIL]);
+		failed_send_cb(sock, uvreq, ISC_R_CANCELED);
+		return;
+	}
+
 	if (sock->tid == isc_nm_tid()) {
 		/*
 		 * If we're in the same thread as the socket we can send the
 		 * data directly
 		 */
-		return (tcp_send_direct(sock, uvreq));
+		isc_result_t result = tcp_send_direct(sock, uvreq);
+		if (result != ISC_R_SUCCESS) {
+			isc__nm_incstats(sock->mgr,
+					 sock->statsindex[STATID_SENDFAIL]);
+			failed_send_cb(sock, uvreq, result);
+		}
 	} else {
 		/*
 		 * We need to create an event and pass it using async channel
@@ -998,32 +1218,28 @@ isc__nm_tcp_send(isc_nmhandle_t *handle, isc_region_t *region, isc_nm_cb_t cb,
 		ievent = isc__nm_get_ievent(sock->mgr, netievent_tcpsend);
 		ievent->sock = sock;
 		ievent->req = uvreq;
+
 		isc__nm_enqueue_ievent(&sock->mgr->workers[sock->tid],
 				       (isc__netievent_t *)ievent);
-		return (ISC_R_SUCCESS);
 	}
-
-	return (ISC_R_UNEXPECTED);
+	return;
 }
 
 static void
 tcp_send_cb(uv_write_t *req, int status) {
 	isc_result_t result = ISC_R_SUCCESS;
 	isc__nm_uvreq_t *uvreq = (isc__nm_uvreq_t *)req->data;
-	isc_nmsocket_t *sock = NULL;
+	isc_nmsocket_t *sock = uvreq->sock;
 
 	REQUIRE(VALID_UVREQ(uvreq));
 	REQUIRE(VALID_NMHANDLE(uvreq->handle));
 
 	if (status < 0) {
 		result = isc__nm_uverr2result(status);
-		isc__nm_incstats(uvreq->sock->mgr,
-				 uvreq->sock->statsindex[STATID_SENDFAIL]);
+		isc__nm_incstats(sock->mgr, sock->statsindex[STATID_SENDFAIL]);
 	}
 
 	uvreq->cb.send(uvreq->handle, result, uvreq->cbarg);
-
-	sock = uvreq->handle->sock;
 	isc__nm_uvreq_put(&uvreq, sock);
 }
 
@@ -1034,18 +1250,17 @@ void
 isc__nm_async_tcpsend(isc__networker_t *worker, isc__netievent_t *ev0) {
 	isc_result_t result;
 	isc__netievent_tcpsend_t *ievent = (isc__netievent_tcpsend_t *)ev0;
+	isc_nmsocket_t *sock = ievent->sock;
+	isc__nm_uvreq_t *uvreq = ievent->req;
 
+	REQUIRE(sock->type == isc_nm_tcpsocket);
 	REQUIRE(worker->id == ievent->sock->tid);
 
-	if (!atomic_load(&ievent->sock->active)) {
-		return;
-	}
-
-	result = tcp_send_direct(ievent->sock, ievent->req);
+	result = tcp_send_direct(sock, uvreq);
 	if (result != ISC_R_SUCCESS) {
-		ievent->req->cb.send(ievent->req->handle, result,
-				     ievent->req->cbarg);
-		isc__nm_uvreq_put(&ievent->req, ievent->req->handle->sock);
+		isc__nm_incstats(sock->mgr, sock->statsindex[STATID_SENDFAIL]);
+		uvreq->cb.send(uvreq->handle, result, uvreq->cbarg);
+		isc__nm_uvreq_put(&uvreq, sock);
 	}
 }
 
@@ -1053,14 +1268,18 @@ static isc_result_t
 tcp_send_direct(isc_nmsocket_t *sock, isc__nm_uvreq_t *req) {
 	int r;
 
+	REQUIRE(VALID_NMSOCK(sock));
+	REQUIRE(VALID_UVREQ(req));
 	REQUIRE(sock->tid == isc_nm_tid());
 	REQUIRE(sock->type == isc_nm_tcpsocket);
+
+	if (inactive(sock)) {
+		return (ISC_R_CANCELED);
+	}
+
 	r = uv_write(&req->uv_req.write, &sock->uv_handle.stream, &req->uvbuf,
 		     1, tcp_send_cb);
 	if (r < 0) {
-		isc__nm_incstats(sock->mgr, sock->statsindex[STATID_SENDFAIL]);
-		req->cb.send(NULL, isc__nm_uverr2result(r), req->cbarg);
-		isc__nm_uvreq_put(&req, sock);
 		return (isc__nm_uverr2result(r));
 	}
 
@@ -1076,19 +1295,19 @@ tcp_close_cb(uv_handle_t *uvhandle) {
 	isc__nm_incstats(sock->mgr, sock->statsindex[STATID_CLOSE]);
 	atomic_store(&sock->closed, true);
 	atomic_store(&sock->connected, false);
+
+	if (sock->server != NULL) {
+		isc__nmsocket_detach(&sock->server);
+	}
+
 	isc__nmsocket_prep_destroy(sock);
 }
 
 static void
 timer_close_cb(uv_handle_t *uvhandle) {
-	isc_nmsocket_t *sock = uv_handle_get_data(uvhandle);
+	uv_handle_t *handle = uv_handle_get_data(uvhandle);
 
-	REQUIRE(VALID_NMSOCK(sock));
-
-	if (sock->server != NULL) {
-		isc__nmsocket_detach(&sock->server);
-	}
-	uv_close(&sock->uv_handle.handle, tcp_close_cb);
+	uv_close(handle, tcp_close_cb);
 }
 
 static void
@@ -1096,17 +1315,29 @@ tcp_close_direct(isc_nmsocket_t *sock) {
 	REQUIRE(VALID_NMSOCK(sock));
 	REQUIRE(sock->tid == isc_nm_tid());
 	REQUIRE(sock->type == isc_nm_tcpsocket);
+
 	if (sock->quota != NULL) {
 		isc_quota_detach(&sock->quota);
 	}
+
+	uv_read_stop((uv_stream_t *)&sock->uv_handle.handle);
+
+	if (sock->timer_running) {
+		uv_timer_stop(&sock->timer);
+		sock->timer_running = false;
+	}
+
 	if (sock->timer_initialized) {
 		sock->timer_initialized = false;
-		uv_timer_stop(&sock->timer);
+		/*
+		 * The read and timer is stopped and the socket will be
+		 * scheduled to be closed, so we can override the data that the
+		 * timer handle holds.
+		 */
+		uv_handle_set_data((uv_handle_t *)&sock->timer,
+				   &sock->uv_handle.handle);
 		uv_close((uv_handle_t *)&sock->timer, timer_close_cb);
 	} else {
-		if (sock->server != NULL) {
-			isc__nmsocket_detach(&sock->server);
-		}
 		uv_close(&sock->uv_handle.handle, tcp_close_cb);
 	}
 }
@@ -1115,6 +1346,12 @@ void
 isc__nm_tcp_close(isc_nmsocket_t *sock) {
 	REQUIRE(VALID_NMSOCK(sock));
 	REQUIRE(sock->type == isc_nm_tcpsocket);
+	REQUIRE(!isc__nmsocket_active(sock));
+
+	if (!atomic_compare_exchange_strong(&sock->closing, &(bool){ false },
+					    true)) {
+		return;
+	}
 
 	if (sock->tid == isc_nm_tid()) {
 		tcp_close_direct(sock);
@@ -1134,10 +1371,14 @@ isc__nm_tcp_close(isc_nmsocket_t *sock) {
 void
 isc__nm_async_tcpclose(isc__networker_t *worker, isc__netievent_t *ev0) {
 	isc__netievent_tcpclose_t *ievent = (isc__netievent_tcpclose_t *)ev0;
+	isc_nmsocket_t *sock = ievent->sock;
 
-	REQUIRE(worker->id == ievent->sock->tid);
+	REQUIRE(VALID_NMSOCK(sock));
+	REQUIRE(sock->tid == isc_nm_tid());
 
-	tcp_close_direct(ievent->sock);
+	UNUSED(worker);
+
+	tcp_close_direct(sock);
 }
 
 void
@@ -1145,23 +1386,36 @@ isc__nm_tcp_shutdown(isc_nmsocket_t *sock) {
 	REQUIRE(VALID_NMSOCK(sock));
 	REQUIRE(sock->tid == isc_nm_tid());
 
-	if (sock->type == isc_nm_tcpsocket && sock->statichandle != NULL) {
-		isc_nm_recv_cb_t cb;
-		void *cbarg;
+	if (sock->type != isc_nm_tcpsocket) {
+		return;
+	}
 
-		cb = sock->recv_cb;
-		cbarg = sock->recv_cbarg;
-		isc__nmsocket_clearcb(sock);
+	if (atomic_load(&sock->connecting)) {
+		return;
+	}
 
-		if (cb != NULL) {
-			cb(sock->statichandle, ISC_R_CANCELED, NULL, cbarg);
-		}
+	/*
+	 * If the socket is active, mark it inactive and
+	 * continue. If it isn't active, stop now.
+	 */
+	if (!isc__nmsocket_deactivate(sock)) {
+		return;
+	}
+
+	if (sock->accepting) {
+		failed_accept_cb(sock, ISC_R_CANCELED);
+		return;
+	}
+
+	if (sock->statichandle != NULL) {
+		failed_read_cb(sock, ISC_R_CANCELED);
 	}
 }
 
 void
 isc__nm_tcp_cancelread(isc_nmhandle_t *handle) {
 	isc_nmsocket_t *sock = NULL;
+	isc__netievent_tcpcancel_t *ievent = NULL;
 
 	REQUIRE(VALID_NMHANDLE(handle));
 
@@ -1169,16 +1423,44 @@ isc__nm_tcp_cancelread(isc_nmhandle_t *handle) {
 
 	REQUIRE(VALID_NMSOCK(sock));
 	REQUIRE(sock->type == isc_nm_tcpsocket);
+
+	ievent = isc__nm_get_ievent(sock->mgr, netievent_tcpcancel);
+	ievent->sock = sock;
+	isc_nmhandle_attach(handle, &ievent->handle);
+	isc__nm_enqueue_ievent(&sock->mgr->workers[sock->tid],
+			       (isc__netievent_t *)ievent);
+}
+
+void
+isc__nm_async_tcpcancel(isc__networker_t *worker, isc__netievent_t *ev0) {
+	isc__netievent_tcpcancel_t *ievent = (isc__netievent_tcpcancel_t *)ev0;
+	isc_nmsocket_t *sock = ievent->sock;
+	isc_nmhandle_t *handle = ievent->handle;
+
+	REQUIRE(VALID_NMSOCK(sock));
+	REQUIRE(worker->id == sock->tid);
 	REQUIRE(sock->tid == isc_nm_tid());
 
+	uv_read_stop(&sock->uv_handle.stream);
+
 	if (atomic_load(&sock->client)) {
-		isc_nm_recv_cb_t cb;
-		void *cbarg;
+		failed_read_cb(sock, ISC_R_EOF);
+	}
 
-		cb = sock->recv_cb;
-		cbarg = sock->recv_cbarg;
-		isc__nmsocket_clearcb(sock);
+	isc_nmhandle_detach(&handle);
+}
 
-		cb(handle, ISC_R_EOF, NULL, cbarg);
+void
+isc__nm_tcp_settimeout(isc_nmhandle_t *handle, uint32_t timeout) {
+	isc_nmsocket_t *sock = NULL;
+
+	REQUIRE(VALID_NMHANDLE(handle));
+
+	sock = handle->sock;
+
+	sock->read_timeout = timeout;
+	if (sock->timer_running) {
+		uv_timer_start(&sock->timer, readtimeout_cb, sock->read_timeout,
+			       0);
 	}
 }
