@@ -1,8 +1,6 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * SPDX-License-Identifier: MPL-2.0
- *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, you can obtain one at https://mozilla.org/MPL/2.0/.
@@ -29,10 +27,8 @@
  */
 isc_result_t
 isc___nm_uverr2result(int uverr, bool dolog, const char *file,
-		      unsigned int line, const char *func) {
+		      unsigned int line) {
 	switch (uverr) {
-	case 0:
-		return (ISC_R_SUCCESS);
 	case UV_ENOTDIR:
 	case UV_ELOOP:
 	case UV_EINVAL: /* XXX sometimes this is not for files */
@@ -85,17 +81,12 @@ isc___nm_uverr2result(int uverr, bool dolog, const char *file,
 		return (ISC_R_CONNREFUSED);
 	case UV_ECANCELED:
 		return (ISC_R_CANCELED);
-	case UV_EOF:
-		return (ISC_R_EOF);
-	case UV_EMSGSIZE:
-		return (ISC_R_MAXSIZE);
 	default:
 		if (dolog) {
-			UNEXPECTED_ERROR(
-				file, line,
-				"unable to convert libuv "
-				"error code in %s to isc_result: %d: %s",
-				func, uverr, uv_strerror(uverr));
+			UNEXPECTED_ERROR(file, line,
+					 "unable to convert libuv "
+					 "error code to isc_result: %d: %s",
+					 uverr, uv_strerror(uverr));
 		}
 		return (ISC_R_UNEXPECTED);
 	}
