@@ -1,8 +1,6 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * SPDX-License-Identifier: MPL-2.0
- *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, you can obtain one at https://mozilla.org/MPL/2.0/.
@@ -30,8 +28,6 @@
 #include <isc/thread.h>
 #include <isc/util.h>
 
-#include "isctest.h"
-
 static void
 isc_quota_get_set_test(void **state) {
 	UNUSED(state);
@@ -56,29 +52,19 @@ isc_quota_get_set_test(void **state) {
 	isc_quota_destroy(&quota);
 }
 
-static void
-add_quota(isc_quota_t *source, isc_quota_t **target,
-	  isc_result_t expected_result, int expected_used) {
-	isc_result_t result;
-
-	*target = NULL;
-
-	result = isc_quota_attach(source, target);
-	assert_int_equal(result, expected_result);
-
-	switch (expected_result) {
-	case ISC_R_SUCCESS:
-	case ISC_R_SOFTQUOTA:
-		assert_ptr_equal(*target, source);
-		break;
-	default:
-		assert_null(*target);
-		break;
+#define add_quota(quota, quotasp, exp, attached, exp_used)              \
+	{                                                               \
+		*quotasp = NULL;                                        \
+		isc_result_t result = isc_quota_attach(quota, quotasp); \
+		assert_int_equal(result, exp);                          \
+		if (attached) {                                         \
+			assert_ptr_equal(*quotasp, quota);              \
+		} else {                                                \
+			assert_null(*quotasp);                          \
+		}                                                       \
+		assert_int_equal(isc_quota_getused(quota), exp_used);   \
 	}
 
-	assert_int_equal(isc_quota_getused(source), expected_used);
-}
-
 static void
 isc_quota_hard_test(void **state) {
 	isc_quota_t quota;
@@ -89,18 +75,18 @@ isc_quota_hard_test(void **state) {
 	isc_quota_init(&quota, 100);
 
 	for (i = 0; i < 100; i++) {
-		add_quota(&quota, &quotas[i], ISC_R_SUCCESS, i + 1);
+		add_quota(&quota, &quotas[i], ISC_R_SUCCESS, true, i + 1);
 	}
 
-	add_quota(&quota, &quotas[100], ISC_R_QUOTA, 100);
+	add_quota(&quota, &quotas[100], ISC_R_QUOTA, false, 100);
 
 	assert_int_equal(isc_quota_getused(&quota), 100);
 
 	isc_quota_detach(&quotas[0]);
 	assert_null(quotas[0]);
 
-	add_quota(&quota, &quotas[100], ISC_R_SUCCESS, 100);
-	add_quota(&quota, &quotas[101], ISC_R_QUOTA, 100);
+	add_quota(&quota, &quotas[100], ISC_R_SUCCESS, true, 100);
+	add_quota(&quota, &quotas[101], ISC_R_QUOTA, false, 100);
 
 	for (i = 100; i > 0; i--) {
 		isc_quota_detach(&quotas[i]);
@@ -122,13 +108,13 @@ isc_quota_soft_test(void **state) {
 	isc_quota_soft(&quota, 50);
 
 	for (i = 0; i < 50; i++) {
-		add_quota(&quota, &quotas[i], ISC_R_SUCCESS, i + 1);
+		add_quota(&quota, &quotas[i], ISC_R_SUCCESS, true, i + 1);
 	}
 	for (i = 50; i < 100; i++) {
-		add_quota(&quota, &quotas[i], ISC_R_SOFTQUOTA, i + 1);
+		add_quota(&quota, &quotas[i], ISC_R_SOFTQUOTA, true, i + 1);
 	}
 
-	add_quota(&quota, &quotas[i], ISC_R_QUOTA, 100);
+	add_quota(&quota, &quotas[i], ISC_R_QUOTA, false, 100);
 
 	for (i = 99; i >= 0; i--) {
 		isc_quota_detach(&quotas[i]);
@@ -139,7 +125,7 @@ isc_quota_soft_test(void **state) {
 	isc_quota_destroy(&quota);
 }
 
-static atomic_uint_fast32_t cb_calls = 0;
+static atomic_uint_fast32_t cb_calls = ATOMIC_VAR_INIT(0);
 static isc_quota_cb_t cbs[30];
 static isc_quota_t *qp;
 
@@ -253,14 +239,14 @@ typedef struct qthreadinfo {
 	isc_quota_cb_t callbacks[100];
 } qthreadinfo_t;
 
-static atomic_uint_fast32_t g_tnum = 0;
+static atomic_uint_fast32_t g_tnum = ATOMIC_VAR_INIT(0);
 /* at most 10 * 100 quota_detach threads */
 isc_thread_t g_threads[10 * 100];
 
 static void *
 quota_detach(void *quotap) {
 	isc_quota_t *quota = (isc_quota_t *)quotap;
-	isc_test_nap(10000);
+	usleep(10000);
 	isc_quota_detach(&quota);
 	return ((isc_threadresult_t)0);
 }
@@ -352,7 +338,7 @@ main(void) {
 int
 main(void) {
 	printf("1..0 # Skipped: cmocka not available\n");
-	return (SKIPPED_TEST_EXIT_CODE);
+	return (0);
 }
 
 #endif /* if HAVE_CMOCKA */
